/*
 * Copyright (c) 2017-2021 The Linux Foundation. All rights reserved.
 * Copyright (c) 2021-2023 Qualcomm Innovation Center, Inc. All rights reserved.
 *
 *
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * DOC: reg_db.c
 * This file implements QCA regulatory database.
 * Current implementation conforms to database version 42.
 */

#include <qdf_types.h>
#include <qdf_trace.h>
#include <wlan_cmn.h>
#include <reg_services_public_struct.h>
#include "reg_db.h"

#ifndef CONFIG_REG_CLIENT
const struct country_code_to_reg_domain g_all_countries[] = {
	CE(AFGHANISTAN, ETSI1_WORLD, 0, AF, 40, 160, 0, 0),
	CE(ALAND_ISLANDS, ETSI13_WORLD, ETSI1_6G_02, AX, 40, 160, 320, 0),
	CE(ALBANIA, ETSI1_WORLD, 0, AL, 40, 160, 0, 0),
	CE(ALGERIA, APL13_WORLD, 0, DZ, 40, 160, 0, 0),
	CE(AMERICAN_SAMOA, FCC3_FCCA, FCC1_6G_01, AS, 40, 160, 320, 0),
	CE(ANDORRA, ETSI1_WORLD, ETSI1_6G_02, AD, 40, 160, 320, 0),
	CE(ANGUILLA, ETSI1_WORLD, 0, AI, 40, 160, 0, 0),
	CE(ANTIGUA_AND_BARBUDA, FCC17_WORLD, FCC1_6G_01, AG, 40, 160, 320, 0),
	CE(ARGENTINA, APL16_ETSIC, 0, AR, 40, 160, 0, 0),
	CE(ARMENIA, APL4_WORLD, 0, AM, 40, 160, 0, 0),
	CE(ARUBA, ETSI1_WORLD, FCC1_6G_01, AW, 40, 160, 320, 0),
	CE(AUSTRALIA, APL25_ETSIC, ETSI2_6G_0C, AU, 40, 160, 320, 0),
	CE(AUSTRIA, ETSI1_WORLD, 0, AT, 40, 160, 0, 0),
	CE(AZERBAIJAN, ETSI8_WORLD, 0, AZ, 40, 160, 0, 0),
	CE(BAHAMAS, FCC17_WORLD, FCC1_6G_01, BS, 40, 160, 320, 0),
	CE(BAHRAIN, APL15_WORLD, 0, BH, 40, 160, 0, 0),
	CE(BANGLADESH, APL1_WORLD, 0, BD, 40, 160, 0, 0),
	CE(BARBADOS, FCC2_WORLD, 0, BB, 40, 160, 0, 0),
	CE(BELARUS, ETSI1_WORLD, 0, BY, 40, 160, 0, 0),
	CE(BELGIUM, ETSI1_WORLD, ETSI1_6G_02, BE, 40, 160, 320, 0),
	CE(BELIZE, ETSI8_WORLD, 0, BZ, 40, 160, 0, 0),
	CE(BERMUDA, FCC17_FCCA, 0, BM, 40, 160, 0, 0),
	CE(BHUTAN, ETSI1_WORLD, 0, BT, 40, 160, 0, 0),
	CE(BOLIVIA, FCC17_WORLD, 0, BO, 40, 160, 0, 0),
	CE(BOSNIA_HERZ, ETSI1_WORLD, 0, BA, 40, 160, 0, 0),
	CE(BOTSWANA, ETSI20_WORLD, 0, BW, 40, 160, 0, 0),
	CE(BRAZIL, FCC18_ETSIC, FCC1_6G_07, BR, 40, 160, 320, 0),
	CE(BRUNEI_DARUSSALAM, FCC17_WORLD, 0, BN, 40, 160, 0, 0),
	CE(BULGARIA, ETSI1_WORLD, ETSI1_6G_02, BG, 40, 160, 320, 0),
	CE(BURKINA_FASO, FCC17_WORLD, 0, BF, 40, 160, 0, 0),
	CE(BURUNDI, ETSI20_WORLD, 0, BI, 40, 160, 0, 0),
	CE(CAMBODIA, ETSI1_WORLD, 0, KH, 40, 160, 0, 0),
	CE(CAMEROON, ETSI1_WORLD, 0, CM, 40, 160, 0, 0),
	CE(CANADA, FCC6_FCCA, FCC1_6G_05, CA, 40, 160, 320, 0),
	CE(CAYMAN_ISLANDS, FCC17_WORLD, FCC1_6G_01, KY, 40, 160, 320, 0),
	CE(CENTRAL_AFRICA_REPUBLIC, FCC17_WORLD, 0, CF, 40, 40, 0, 0),
	CE(CHAD, ETSI1_WORLD, 0, TD, 40, 160, 0, 0),
	CE(CHILE, FCC20_FCCA, APL7_6G_13, CL, 40, 160, 320, 0),
	CE(CHINA, APL14_CHNA, 0, CN, 40, 160, 0, 0),
	CE(CHRISTMAS_ISLAND, APL25_ETSIC, ETSI2_6G_0C, CX, 40, 320, 0, 0),
	CE(COLOMBIA, FCC17_FCCA, 0, CO, 40, 160, 0, 0),
	CE(CONGO, ETSI12_WORLD, 0, CG, 40, 160, 0, 0),
	CE(CONGO_DEMOCRATIC_REPUBLIC, ETSI20_WORLD, 0, CD, 40, 160, 0, 0),
	CE(COOK_ISLANDS, APL26_ETSIC, ETSI2_6G_0E, CK, 40, 160, 320, 0),
	CE(COSTA_RICA, FCC17_WORLD, APL4_6G_11, CR, 40, 160, 320, 0),
	CE(COTE_DIVOIRE, FCC17_WORLD, 0, CI, 40, 160, 0, 0),
	CE(CROATIA, ETSI1_WORLD, 0, HR, 40, 160, 0, 0),
	CE(CURACAO, APL10_WORLD, ETSI1_6G_02, CW, 40, 160, 320, 0),
	CE(CYPRUS, ETSI1_WORLD, ETSI1_6G_02, CY, 40, 160, 320, 0),
	CE(CZECH, ETSI1_WORLD, ETSI1_6G_02, CZ, 40, 160, 320, 0),
	CE(DENMARK, ETSI1_WORLD, ETSI1_6G_02, DK, 40, 160, 320, 0),
	CE(DOMINICA, FCC1_FCCA, 0, DM, 40, 160, 0, 0),
	CE(DOMINICAN_REPUBLIC, FCC17_FCCA, FCC1_6G_14, DO, 40, 160, 320, 0),
	CE(ECUADOR, FCC17_FCCA, 0, EC, 40, 160, 0, 0),
	CE(EGYPT, ETSI3_WORLD, 0, EG, 40, 160, 0, 0),
	CE(EL_SALVADOR, FCC17_WORLD, FCC1_6G_01, SV, 40, 160, 320, 0),
	CE(ESTONIA, ETSI1_WORLD, ETSI1_6G_02, EE, 40, 160, 320, 0),
	CE(ETHIOPIA, ETSI1_WORLD, 0, ET, 40, 160, 0, 0),
	CE(FALKLAND_ISLANDS, ETSI1_WORLD, ETSI2_6G_03, FK, 40, 160, 320, 0),
	CE(FAROE_ISLANDS, ETSI1_WORLD, ETSI1_6G_02, FO, 40, 160, 320, 0),
	CE(FIJI, APL1_WORLD, ETSI2_6G_0C, FJ, 40, 160, 320, 0),
	CE(FINLAND, ETSI1_WORLD, ETSI1_6G_02, FI, 40, 160, 320, 0),
	CE(FRANCE, ETSI1_WORLD, ETSI1_6G_02, FR, 40, 160, 320, 0),
	CE(FRENCH_GUIANA, ETSI1_WORLD, ETSI1_6G_02, GF, 40, 160, 320, 0),
	CE(FRENCH_POLYNESIA, ETSI1_WORLD, ETSI1_6G_02, PF, 40, 160, 320, 0),
	CE(FRENCH_SOUTHERN_TERRITORIES, ETSI1_WORLD, ETSI1_6G_02, TF, 40, 160,
	   320, 0),
	CE(GABON, ETSI3_WORLD, 0, GA, 40, 160, 0, 0),
	CE(GEORGIA, ETSI1_WORLD, ETSI1_6G_02, GE, 40, 160, 320, 0),
	CE(GERMANY, ETSI1_WORLD, ETSI1_6G_02, DE, 40, 160, 320, 0),
	CE(GHANA, FCC17_WORLD, 0, GH, 40, 160, 0, 0),
	CE(GIBRALTAR, ETSI1_WORLD, ETSI2_6G_03, GI, 40, 160, 320, 0),
	CE(GREECE, ETSI1_WORLD,  ETSI1_6G_02, GR, 40, 160, 320, 0),
	CE(GREENLAND, ETSI1_WORLD, 0, GL, 40, 160, 0, 0),
	CE(GRENADA, FCC17_FCCA, FCC1_6G_01, GD, 40, 160, 320, 0),
	CE(GUADELOUPE, ETSI1_WORLD, ETSI1_6G_02, GP, 40, 160, 320, 0),
	CE(GUAM, FCC3_FCCA, FCC1_6G_01, GU, 40, 160, 320, 0),
	CE(GUATEMALA, APL29_ETSIC, APL2_6G_06, GT, 40, 160, 320, 0),
	CE(GUERNSEY, ETSI1_WORLD, ETSI2_6G_03, GG, 40, 160, 320, 0),
	CE(GUYANA, APL1_ETSIC, 0, GY, 40, 160, 0, 0),
	CE(HAITI, FCC17_FCCA, 0, HT, 40, 160, 0, 0),
	CE(HEARD_ISLAND_AND_MCDONALD_ISLANDS, APL25_ETSIC, ETSI2_6G_0C, HM, 40,
	   160, 320, 0),
	CE(HOLY_SEE, ETSI1_WORLD, ETSI1_6G_02, VA, 40, 160, 320, 0),
	CE(HONDURAS, FCC17_WORLD, FCC1_6G_07, HN, 40, 160, 320, 0),
	CE(HONG_KONG, APL20_WORLD, ETSI2_6G_03, HK, 40, 160, 320, 0),
	CE(HUNGARY, ETSI1_WORLD, ETSI1_6G_02, HU, 40, 160, 320, 0),
	CE(ICELAND, ETSI1_WORLD, ETSI1_6G_02, IS, 40, 160, 320, 0),
	CE(INDIA, APL19_ETSIC, 0, IN, 40, 160, 0, 0),
	CE(INDONESIA, APL2_ETSID, 0, ID, 40, 80, 0, 0),
	CE(IRAQ, ETSI1_WORLD, 0, IQ, 40, 160, 0, 0),
	CE(IRELAND, ETSI1_WORLD, ETSI1_6G_02, IE, 40, 160, 320, 0),
	CE(ISLE_OF_MAN, ETSI1_WORLD, ETSI2_6G_03, IM, 40, 160, 320, 0),
	CE(ISRAEL, ETSI17_WORLD, ETSI1_6G_0D, IL, 40, 160, 320, 0),
	CE(ITALY, ETSI1_WORLD, ETSI1_6G_02, IT, 40, 160, 320, 0),
	CE(JAMAICA, FCC13_WORLD, FCC1_6G_01, JM, 40, 160, 320, 0),
	CE(JAPAN, MKK17_MKKC, MKK1_6G_0B, JP, 40, 160, 320, 0),
	CE(JAPAN15, MKK17_MKKC, 0, JP, 40, 160, 0, 0),
	CE(JERSEY, ETSI1_WORLD, ETSI2_6G_03, JE, 40, 160, 320, 0),
	CE(JORDAN, APL4_WORLD, ETSI1_6G_02, JO, 40, 160, 320, 0),
	CE(KAZAKHSTAN, MKK5_MKKC, 0, KZ, 40, 160, 0, 0),
	CE(KENYA, ETSI20_WORLD, ETSI1_6G_02, KE, 40, 160, 320, 0),
	CE(KOREA_ROC, APL9_KRRA, APL1_6G_04, KR, 40, 160, 160, 0),
	CE(KUWAIT, ETSI1_WORLD, ETSI1_6G_0D, KW, 40, 160, 320, 0),
	CE(LAO_PEOPLES_DEMOCRATIC_REPUBLIC, ETSI20_WORLD, FCC1_6G_05, LA, 40,
	   160, 320, 0),
	CE(LATVIA, ETSI1_WORLD,  ETSI1_6G_02, LV, 40, 160, 320, 0),
	CE(LEBANON, FCC17_WORLD, 0, LB, 40, 160, 0, 0),
	CE(LESOTHO, ETSI1_WORLD, 0, LS, 40, 160, 0, 0),
	CE(LIBYA, ETSI12_WORLD, 0, LY, 40, 160, 0, 0),
	CE(LIECHTENSTEIN, ETSI1_WORLD, ETSI1_6G_02, LI, 40, 160, 320, 0),
	CE(LITHUANIA, ETSI1_WORLD, ETSI1_6G_02, LT, 40, 160, 320, 0),
	CE(LUXEMBOURG, ETSI1_WORLD,  ETSI1_6G_02, LU, 40, 160, 320, 0),
	CE(MACAU, FCC17_WORLD, 0, MO, 40, 160, 0, 0),
	CE(MACEDONIA, ETSI1_WORLD, 0, MK, 40, 160, 0, 0),
	CE(MALAWI, ETSI1_WORLD, 0, MW, 40, 160, 0, 0),
	CE(MALAYSIA, FCC11_WORLD, ETSI1_6G_02, MY, 40, 160, 320, 0),
	CE(MALDIVES, APL6_WORLD, 0, MV, 40, 160, 0, 0),
	CE(MALTA, ETSI1_WORLD, ETSI1_6G_02, MT, 40, 160, 320, 0),
	CE(MARSHALL_ISLANDS, FCC17_FCCA, 0, MH, 40, 160, 0, 0),
	CE(MARTINIQUE, ETSI1_WORLD, ETSI1_6G_02, MQ, 40, 160, 320, 0),
	CE(MAURITANIA, ETSI1_WORLD, 0, MR, 40, 160, 0, 0),
	CE(MAURITIUS, ETSI1_WORLD, ETSI1_6G_02, MU, 40, 160, 320, 0),
	CE(MAYOTTE, ETSI1_WORLD, ETSI1_6G_02, YT, 40, 160, 320, 0),
	CE(MEXICO, APL16_ETSIC, 0, MX, 40, 160, 0, 0),
	CE(MICRONESIA, FCC17_FCCA, 0, FM, 40, 160, 0, 0),
	CE(MOLDOVA, ETSI1_WORLD, 0, MD, 40, 160, 0, 0),
	CE(MONACO, ETSI1_WORLD, 0, MC, 40, 160, 0, 0),
	CE(MONGOLIA, FCC17_WORLD, ETSI1_6G_02, MN, 40, 160, 320, 0),
	CE(MONTENEGRO, ETSI1_WORLD, 0, ME, 40, 160, 0, 0),
	CE(MONTSERRAT, ETSI1_WORLD, ETSI2_6G_03, MS, 40, 160, 320, 0),
	CE(MOROCCO, ETSI3_WORLD, ETSI1_6G_02, MA, 40, 160, 320, 0),
	CE(MYANMAR, APL1_WORLD, 0, MM, 40, 160, 0, 0),
	CE(NAMIBIA, APL20_WORLD, 0, NA, 40, 160, 0, 0),
	CE(NEPAL, APL23_WORLD, 0, NP, 40, 160, 0, 0),
	CE(NETHERLANDS, ETSI1_WORLD, ETSI1_6G_02, NL, 40, 160, 320, 0),
	CE(NETHERLANDS_ANTILLES, ETSI1_WORLD, ETSI1_6G_02, AN, 40, 160, 320, 0),
	CE(NEW_CALEDONIA, ETSI1_WORLD, ETSI1_6G_02, NC, 40, 160, 320, 0),
	CE(NEW_ZEALAND, APL26_ETSIC, ETSI2_6G_0E, NZ, 40, 160, 320, 0),
	CE(NICARAGUA, APL27_FCCA, 0, NI, 40, 160, 0, 0),
	CE(NIGERIA, APL6_WORLD, 0, NG, 40, 160, 0, 0),
	CE(NIUE, APL26_ETSIC, ETSI2_6G_0E, NU, 40, 160, 320, 0),
	CE(NORFOLK_ISLAND, APL25_ETSIC, ETSI2_6G_0C, NF, 40, 160, 320, 0),
	CE(NORTHERN_MARIANA_ISLANDS, FCC3_FCCA, FCC1_6G_01, MP, 40, 160, 320,
	   0),
	CE(NORWAY, ETSI1_WORLD, ETSI1_6G_02, NO, 40, 160, 320, 0),
	CE(OMAN, ETSI1_WORLD, 0, OM, 40, 160, 0, 0),
	CE(PAKISTAN, APL28_ETSIC, 0, PK, 40, 160, 0, 0),
	CE(PALAU, FCC17_FCCA, 0, PW, 40, 160, 0, 0),
	CE(PANAMA, FCC14_FCCB, 0, PA, 40, 160, 0, 0),
	CE(PAPUA_NEW_GUINEA, FCC17_WORLD, 0, PG, 40, 160, 0, 0),
	CE(PARAGUAY, FCC17_WORLD, 0, PY, 40, 160, 0, 0),
	CE(PERU, FCC17_WORLD, FCC1_6G_05, PE, 40, 160, 320, 0),
	CE(PHILIPPINES, ETSI9_WORLD, 0, PH, 40, 160, 0, 0),
	CE(POLAND, ETSI1_WORLD,  ETSI1_6G_02, PL, 40, 160, 320, 0),
	CE(PORTUGAL, ETSI1_WORLD, ETSI1_6G_02, PT, 40, 160, 320, 0),
	CE(PUERTO_RICO, FCC3_FCCA, FCC1_6G_01, PR, 40, 160, 320, 0),
	CE(QATAR, ETSI14_WORLD, ETSI1_6G_02, QA, 40, 160, 320, 0),
	CE(REUNION, ETSI1_WORLD, ETSI1_6G_02, RE, 40, 160, 320, 0),
	CE(ROMANIA, ETSI1_WORLD, 0, RO, 40, 160, 0, 0),
	CE(RUSSIA, ETSI8_WORLD, 0, RU, 40, 160, 0, 0),
	CE(RWANDA, ETSI20_WORLD, 0, RW, 40, 160, 0, 0),
	CE(SAINT_BARTHELEMY, ETSI1_WORLD, 0, BL, 40, 160, 0, 0),
	CE(SAINT_HELENA_ASCENSION_AND_TRISTAN_DA_CUNHA, ETSI1_WORLD,
	   ETSI2_6G_03, SH, 40, 160, 320, 0),
	CE(SAINT_KITTS_AND_NEVIS, APL10_WORLD, 0, KN, 40, 160, 0, 0),
	CE(SAINT_LUCIA, APL10_WORLD, 0, LC, 40, 160, 0, 0),
	CE(SAINT_MARTIN, ETSI1_WORLD, 0, MF, 40, 160, 0, 0),
	CE(SAINT_PIERRE_AND_MIQUELON, ETSI1_WORLD, ETSI1_6G_02, PM, 40, 160,
	   320, 0),
	CE(SAINT_VINCENT_AND_THE_GRENADIENS, ETSI1_WORLD, 0, VC,
	   40, 160, 0, 0),
	CE(SAMOA, ETSI1_WORLD, 0, WS, 40, 40, 0, 0),
	CE(SAN_MARINO, ETSI1_WORLD,  ETSI1_6G_02, SM, 40, 160, 320, 0),
	CE(SAO_TOME_AND_PRINCIPE, FCC17_WORLD, 0, ST, 40, 160, 0, 0),
	CE(SAUDI_ARABIA, ETSI20_WORLD, APL6_6G_0A, SA, 40, 160, 320, 0),
	CE(SENEGAL, ETSI1_WORLD, 0, SN, 40, 160, 0, 0),
	CE(SERBIA, ETSI1_WORLD, 0, RS, 40, 160, 0, 0),
	CE(SINGAPORE, APL20_WORLD, 0, SG, 40, 160, 0, 0),
	CE(SINT_MAARTEN, ETSI1_WORLD, ETSI1_6G_02, SX, 40, 160, 320, 0),
	CE(SLOVAKIA, ETSI1_WORLD,  ETSI1_6G_02, SK, 40, 160, 320, 0),
	CE(SLOVENIA, ETSI1_WORLD,  ETSI1_6G_02, SI, 40, 160, 320, 0),
	CE(SOUTH_AFRICA, ETSI20_WORLD, 0, ZA, 40, 160, 0, 0),
	CE(SPAIN, ETSI1_WORLD, ETSI1_6G_02, ES, 40, 160, 320, 0),
	CE(SRI_LANKA, ETSI15_ETSIC,  ETSI1_6G_02, LK, 40, 160, 320, 0),
	CE(SURINAME, ETSI1_WORLD, 0, SR, 40, 160, 0, 0),
	CE(SVALBARD_AND_JAN_MAYEN, ETSI1_WORLD, ETSI1_6G_02, SJ, 40, 160, 320,
	   0),
	CE(SWEDEN, ETSI1_WORLD, ETSI1_6G_02, SE, 40, 160, 320, 0),
	CE(SWITZERLAND, ETSI1_WORLD, ETSI1_6G_02, CH, 40, 160, 320, 0),
	CE(TAIWAN, FCC17_FCCA, 0, TW, 40, 160, 0, 0),
	CE(TANZANIA, ETSI15_WORLD, 0, TZ, 40, 160, 0, 0),
	CE(THAILAND, APL20_WORLD, 0, TH, 40, 160, 0, 0),
	CE(TOGO, ETSI1_WORLD, 0, TG, 40, 40, 0, 0),
	CE(TRINIDAD_Y_TOBAGO, FCC17_WORLD, 0, TT, 40, 160, 0, 0),
	CE(TUNISIA, ETSI3_WORLD, 0, TN, 40, 160, 0, 0),
	CE(TURKEY, ETSI1_WORLD,  ETSI1_6G_02, TR, 40, 160, 320, 0),
	CE(TURKS_AND_CAICOS, FCC17_WORLD, 0, TC, 40, 160, 0, 0),
	CE(UGANDA, FCC17_WORLD, 0, UG, 40, 160, 0, 0),
	CE(UKRAINE, ETSI9_WORLD, 0, UA, 40, 160, 0, 0),
	CE(UAE, ETSI20_WORLD, APL3_6G_08, AE, 40, 160, 320, 0),
	CE(UNITED_KINGDOM, ETSI1_WORLD, ETSI2_6G_03, GB, 40, 160, 320, 0),
	CE(UNITED_STATES, FCC8_FCCA, FCC1_6G_01, US, 40, 160, 320, 0),
	CE(UNITED_STATES_MINOR_OUTLYING_ISLANDS, FCC8_FCCA, FCC1_6G_01, UM, 40,
	   160, 320, 0),
	CE(URUGUAY, FCC2_WORLD, 0, UY, 40, 160, 0, 0),
	CE(UZBEKISTAN, ETSI3_WORLD, 0, UZ, 40, 160, 0, 0),
	CE(VANUATU, FCC17_WORLD, 0, VU, 40, 160, 0, 0),
	CE(VENEZUELA, FCC2_ETSIC, FCC1_6G_01, VE, 40, 160, 320, 0),
	CE(VIET_NAM, ETSI15_WORLD, 0, VN, 40, 160, 0, 0),
	CE(VIRGIN_ISLANDS, FCC3_FCCA, FCC1_6G_01, VI, 40, 160, 320, 0),
	CE(VIRGIN_ISLANDS_BRITISH, ETSI1_WORLD, ETSI2_6G_03, VG, 40, 160, 320,
	   0),
	CE(WALLIS_AND_FUTUNA, ETSI1_WORLD, 0, WF, 40, 160, 0, 0),
	CE(YEMEN, ETSI1_WORLD, 0, YE, 40, 160, 0, 0),
	CE(ZAMBIA, ETSI20_WORLD, 0, ZM, 40, 160, 0, 0),
	CE(ZIMBABWE, ETSI1_WORLD, 0, ZW, 40, 160, 0, 0),
};
#else
#ifdef WLAN_FEATURE_DSRC
const struct country_code_to_reg_domain g_all_countries[] = {
	CE(AFGHANISTAN, ETSI1_WORLD, 0, AF, 40, 160, 0, 0),
	CE(ALAND_ISLANDS, FCC17_WORLD, ETSI1_6G_02, AX, 40, 160, 320, 0),
	CE(ALBANIA, ETSI20_WORLD, 0, AL, 40, 160, 0, 0),
	CE(ALGERIA, APL13_WORLD, 0, DZ, 40, 160, 0, 0),
	CE(AMERICAN_SAMOA, FCC3_FCCA, FCC1_6G_01, AS, 40, 160, 320, 0),
	CE(ANDORRA, ETSI1_WORLD, ETSI1_6G_02, AD, 40, 160, 320, 0),
	CE(ANGUILLA, ETSI1_WORLD, 0, AI, 40, 160, 0, 0),
	CE(ANTIGUA_AND_BARBUDA, ETSI10_WORLD, FCC1_6G_01, AG, 40, 160, 320, 0),
	CE(ARGENTINA, APL17_ETSIC, 0, AR, 40, 160, 0, 0),
	CE(ARMENIA, APL4_WORLD, 0, AM, 40, 160, 0, 0),
	CE(ARUBA, ETSI1_WORLD, FCC1_6G_01, AW, 40, 160, 320, 0),
	CE(AUSTRALIA, APL25_ETSIC, ETSI2_6G_0C, AU, 40, 160, 320, 0),
	CE(AUSTRIA, ETSI10_WORLD, 0, AT, 40, 160, 0, 0),
	CE(AZERBAIJAN, ETSI8_WORLD, 0, AZ, 40, 160, 0, 0),
	CE(BAHAMAS, FCC17_WORLD, FCC1_6G_01, BS, 40, 160, 320, 0),
	CE(BAHRAIN, APL15_WORLD, 0, BH, 40, 160, 0, 0),
	CE(BANGLADESH, APL1_WORLD, 0, BD, 40, 160, 0, 0),
	CE(BARBADOS, FCC2_WORLD, 0, BB, 40, 160, 0, 0),
	CE(BELARUS, ETSI1_WORLD, 0, BY, 40, 160, 0, 0),
	CE(BELGIUM, ETSI10_WORLD, ETSI1_6G_02, BE, 40, 160, 320, 0),
	CE(BELIZE, ETSI8_WORLD, 0, BZ, 40, 160, 0, 0),
	CE(BERMUDA, FCC17_FCCA, 0, BM, 40, 160, 0, 0),
	CE(BHUTAN, ETSI1_WORLD, 0, BT, 40, 160, 0, 0),
	CE(BOLIVIA, FCC17_WORLD, 0, BO, 40, 160, 0, 0),
	CE(BOSNIA_HERZ, ETSI20_WORLD, 0, BA, 40, 160, 0, 0),
	CE(BOTSWANA, ETSI20_WORLD, 0, BW, 40, 160, 0, 0),
	CE(BRAZIL, FCC18_ETSIC, FCC1_6G_07, BR, 40, 160, 320, 0),
	CE(BRUNEI_DARUSSALAM, FCC17_WORLD, 0, BN, 40, 160, 0, 0),
	CE(BULGARIA, ETSI10_WORLD, ETSI1_6G_02, BG, 40, 160, 320, 0),
	CE(BURKINA_FASO, FCC17_WORLD, 0, BF, 40, 160, 0, 0),
	CE(BURUNDI, ETSI20_WORLD, 0, BI, 40, 160, 0, 0),
	CE(CAMBODIA, ETSI1_WORLD, 0, KH, 40, 160, 0, 0),
	CE(CAMEROON, ETSI1_WORLD, 0, CM, 40, 160, 0, 0),
	CE(CANADA, FCC3_FCCA, FCC1_6G_05, CA, 40, 160, 320, 0),
	CE(CAYMAN_ISLANDS, FCC17_WORLD, FCC1_6G_01, KY, 40, 160, 320, 0),
	CE(CENTRAL_AFRICA_REPUBLIC, FCC17_WORLD, 0, CF, 40, 40, 0, 0),
	CE(CHAD, ETSI1_WORLD, 0, TD, 40, 160, 0, 0),
	CE(CHILE, FCC20_FCCA, APL7_6G_13, CL, 40, 160, 320, 0),
	CE(CHINA, APL14_CHNA, 0, CN, 40, 160, 0, 0),
	CE(CHRISTMAS_ISLAND, APL25_ETSIC, ETSI2_6G_0C, CX, 40, 160, 320, 0),
	CE(COLOMBIA, FCC17_WORLD, 0, CO, 40, 160, 0, 0),
	CE(CONGO, ETSI12_WORLD, 0, CG, 40, 160, 0, 0),
	CE(CONGO_DEMOCRATIC_REPUBLIC, ETSI20_WORLD, 0, CD, 40, 160, 0, 0),
	CE(COOK_ISLANDS, APL26_ETSIC, ETSI2_6G_0E, CK, 40, 160, 320, 0),
	CE(COSTA_RICA, FCC17_WORLD, APL4_6G_11, CR, 40, 160, 320, 0),
	CE(COTE_DIVOIRE, FCC17_WORLD, 0, CI, 40, 160, 0, 0),
	CE(CROATIA, ETSI10_WORLD, 0, HR, 40, 160, 0, 0),
	CE(CURACAO, APL10_WORLD, ETSI1_6G_02, CW, 40, 160, 320, 0),
	CE(CYPRUS, ETSI10_WORLD, ETSI1_6G_02, CY, 40, 160, 320, 0),
	CE(CZECH, ETSI10_WORLD, ETSI1_6G_02, CZ, 40, 160, 320, 0),
	CE(DENMARK, ETSI10_WORLD, ETSI1_6G_02, DK, 40, 160, 320, 0),
	CE(DOMINICA, FCC2_FCCA, 0, DM, 40, 160, 0, 0),
	CE(DOMINICAN_REPUBLIC, FCC17_FCCA, FCC1_6G_14, DO, 40, 160, 320, 0),
	CE(ECUADOR, FCC17_FCCA, 0, EC, 40, 160, 0, 0),
	CE(EGYPT, ETSI3_WORLD, 0, EG, 40, 160, 0, 0),
	CE(EL_SALVADOR, FCC17_WORLD, FCC1_6G_01, SV, 40, 160, 320, 0),
	CE(ESTONIA, ETSI10_WORLD, ETSI1_6G_02, EE, 40, 160, 320, 0),
	CE(ETHIOPIA, ETSI1_WORLD, 0, ET, 40, 160, 0, 0),
	CE(FALKLAND_ISLANDS, ETSI10_WORLD, ETSI2_6G_03, FK, 40, 160, 320, 0),
	CE(FAROE_ISLANDS, ETSI10_WORLD, ETSI1_6G_02, FO, 40, 160, 320, 0),
	CE(FIJI, APL1_WORLD, ETSI2_6G_0C, FJ, 40, 160, 320, 0),
	CE(FINLAND, ETSI10_WORLD, ETSI1_6G_02, FI, 40, 160, 320, 0),
	CE(FRANCE, ETSI10_WORLD, ETSI1_6G_02, FR, 40, 160, 320, 0),
	CE(FRENCH_GUIANA, ETSI10_WORLD, ETSI1_6G_02, GF, 40, 160, 320, 0),
	CE(FRENCH_POLYNESIA, ETSI10_WORLD, ETSI1_6G_02, PF, 40, 160, 320, 0),
	CE(FRENCH_SOUTHERN_TERRITORIES, ETSI10_WORLD, ETSI1_6G_02, TF, 40, 160,
	   320, 0),
	CE(GABON, ETSI3_WORLD, 0, GA, 40, 160, 0, 0),
	CE(GEORGIA, ETSI1_WORLD, ETSI1_6G_02, GE, 40, 160, 320, 0),
	CE(GERMANY, ETSI10_WORLD, ETSI1_6G_02, DE, 40, 160, 320, 0),
	CE(GHANA, FCC17_WORLD, 0, GH, 40, 160, 0, 0),
	CE(GIBRALTAR, ETSI10_WORLD, ETSI2_6G_03, GI, 40, 160, 320, 0),
	CE(GREECE, ETSI10_WORLD,  ETSI1_6G_02, GR, 40, 160, 320, 0),
	CE(GREENLAND, ETSI10_WORLD, 0, GL, 40, 160, 0, 0),
	CE(GRENADA, FCC17_FCCA, FCC1_6G_01, GD, 40, 160, 320, 0),
	CE(GUADELOUPE, ETSI1_WORLD, ETSI1_6G_02, GP, 40, 160, 320, 0),
	CE(GUAM, FCC10_FCCA, FCC1_6G_01, GU, 40, 160, 320, 0),
	CE(GUATEMALA, APL29_ETSIC, APL2_6G_06, GT, 40, 160, 320, 0),
	CE(GUERNSEY, ETSI10_WORLD, ETSI2_6G_03, GG, 40, 160, 320, 0),
	CE(GUYANA, APL1_ETSIC, 0, GY, 40, 160, 0, 0),
	CE(HAITI, FCC17_FCCA, 0, HT, 40, 160, 0, 0),
	CE(HEARD_ISLAND_AND_MCDONALD_ISLANDS, APL25_ETSIC, ETSI2_6G_0C, HM, 40,
	   160, 320, 0),
	CE(HOLY_SEE, ETSI10_WORLD, ETSI1_6G_02, VA, 40, 160, 320, 0),
	CE(HONDURAS, FCC13_WORLD, FCC1_6G_07, HN, 40, 160, 320, 0),
	CE(HONG_KONG, APL20_WORLD, ETSI2_6G_03, HK, 40, 160, 320, 0),
	CE(HUNGARY, ETSI10_WORLD, ETSI1_6G_02, HU, 40, 160, 320, 0),
	CE(ICELAND, ETSI10_WORLD, ETSI1_6G_02, IS, 40, 160, 320, 0),
	CE(INDIA, APL15_WORLD, 0, IN, 40, 160, 0, 0),
	CE(INDONESIA, APL2_ETSID, 0, ID, 40, 80, 0, 0),
	CE(IRAQ, ETSI1_WORLD, 0, IQ, 40, 160, 0, 0),
	CE(IRELAND, ETSI10_WORLD, ETSI1_6G_02, IE, 40, 160, 320, 0),
	CE(ISLE_OF_MAN, ETSI10_WORLD, ETSI2_6G_03, IM, 40, 160, 320, 0),
	CE(ISRAEL, ETSI3_WORLD, ETSI1_6G_0D, IL, 40, 160, 320, 0),
	CE(ITALY, ETSI10_WORLD,  ETSI1_6G_02, IT, 40, 160, 320, 0),
	CE(JAMAICA, FCC13_WORLD, FCC1_6G_01, JM, 40, 160, 320, 0),
	CE(JAPAN, MKK17_MKKC, MKK1_6G_0B, JP, 40, 160, 320, 0),
	CE(JAPAN15, MKK5_MKKC, 0, JP, 40, 160, 0, 0),
	CE(XA, MKK5_MKKA, 0, XA, 40, 160, 0, 0),
	CE(JERSEY, ETSI10_WORLD, ETSI2_6G_03, JE, 40, 160, 320, 0),
	CE(JORDAN, APL4_WORLD, ETSI1_6G_02, JO, 40, 160, 320, 0),
	CE(KAZAKHSTAN, MKK5_MKKC, 0, KZ, 40, 160, 0, 0),
	CE(KENYA, ETSI20_WORLD, ETSI1_6G_02, KE, 40, 160, 320, 0),
	CE(KOREA_ROC, APL9_KRRA, APL1_6G_04, KR, 40, 160, 160, 0),
	CE(KUWAIT, ETSI13_WORLD, ETSI1_6G_0D, KW, 40, 160, 320, 0),
	CE(LAO_PEOPLES_DEMOCRATIC_REPUBLIC, ETSI20_WORLD, FCC1_6G_05, LA, 40,
	   160, 320, 0),
	CE(LATVIA, ETSI10_WORLD,  ETSI1_6G_02, LV, 40, 160, 320, 0),
	CE(LEBANON, FCC17_WORLD, 0, LB, 40, 160, 0, 0),
	CE(LESOTHO, ETSI1_WORLD, 0, LS, 40, 160, 0, 0),
	CE(LIBYA, ETSI12_WORLD, 0, LY, 40, 160, 0, 0),
	CE(LIECHTENSTEIN, ETSI10_WORLD, ETSI1_6G_02, LI, 40, 160, 320, 0),
	CE(LITHUANIA, ETSI10_WORLD, ETSI1_6G_02, LT, 40, 160, 320, 0),
	CE(LUXEMBOURG, ETSI10_WORLD,  ETSI1_6G_02, LU, 40, 160, 320, 0),
	CE(MACAU, FCC17_WORLD, 0, MO, 40, 160, 0, 0),
	CE(MACEDONIA, ETSI20_WORLD, 0, MK, 40, 160, 0, 0),
	CE(MALAWI, ETSI1_WORLD, 0, MW, 40, 160, 0, 0),
	CE(MALAYSIA, FCC11_WORLD, ETSI1_6G_02, MY, 40, 160, 320, 0),
	CE(MALDIVES, APL6_WORLD, 0, MV, 40, 160, 0, 0),
	CE(MALTA, ETSI10_WORLD, ETSI1_6G_02, MT, 40, 160, 320, 0),
	CE(MARSHALL_ISLANDS, FCC17_FCCA, 0, MH, 40, 160, 0, 0),
	CE(MARTINIQUE, ETSI10_WORLD, ETSI1_6G_02, MQ, 40, 160, 320, 0),
	CE(MAURITANIA, ETSI1_WORLD, 0, MR, 40, 160, 0, 0),
	CE(MAURITIUS, ETSI20_WORLD, ETSI1_6G_02, MU, 40, 160, 320, 0),
	CE(MAYOTTE, ETSI1_WORLD, ETSI1_6G_02, YT, 40, 160, 320, 0),
	CE(MEXICO, APL16_ETSIC, 0, MX, 40, 160, 0, 0),
	CE(MICRONESIA, FCC17_FCCA, 0, FM, 40, 160, 0, 0),
	CE(MOLDOVA, ETSI20_WORLD, 0, MD, 40, 160, 0, 0),
	CE(MONACO, ETSI10_WORLD, 0, MC, 40, 160, 0, 0),
	CE(MONGOLIA, FCC17_WORLD, ETSI1_6G_02, MN, 40, 160, 320, 0),
	CE(MONTENEGRO, ETSI10_WORLD, 0, ME, 40, 160, 0, 0),
	CE(MONTSERRAT, ETSI10_WORLD, ETSI2_6G_03, MS, 40, 160, 320, 0),
	CE(MOROCCO, ETSI3_WORLD, ETSI1_6G_02, MA, 40, 160, 320, 0),
	CE(MYANMAR, APL1_WORLD, 0, MM, 40, 160, 0, 0),
	CE(NAMIBIA, APL20_WORLD, 0, NA, 40, 160, 0, 0),
	CE(NEPAL, APL23_WORLD, 0, NP, 40, 160, 0, 0),
	CE(NETHERLANDS, ETSI10_WORLD, ETSI1_6G_02, NL, 40, 160, 320, 0),
	CE(NETHERLANDS_ANTILLES, ETSI10_WORLD, ETSI1_6G_02, AN, 40, 160, 320,
	   0),
	CE(NEW_CALEDONIA, ETSI10_WORLD, ETSI1_6G_02, NC, 40, 160, 320, 0),
	CE(NEW_ZEALAND, APL26_ETSIC, ETSI2_6G_0E, NZ, 40, 160, 320, 0),
	CE(NICARAGUA, APL27_FCCA, 0, NI, 40, 160, 0, 0),
	CE(NIGERIA, APL6_WORLD, 0, NG, 40, 160, 0, 0),
	CE(NIUE, ETSI10_WORLD, ETSI2_6G_0E, NU, 40, 160, 320, 0),
	CE(NORFOLK_ISLAND, APL25_ETSIC, ETSI2_6G_0C, NF, 40, 160, 320, 0),
	CE(NORTHERN_MARIANA_ISLANDS, FCC10_FCCA, FCC1_6G_01, MP, 40, 160, 320,
	   0),
	CE(NORWAY, ETSI10_WORLD, ETSI1_6G_02, NO, 40, 160, 320, 0),
	CE(OMAN, ETSI1_WORLD, 0, OM, 40, 160, 0, 0),
	CE(PAKISTAN, APL28_ETSIC, 0, PK, 40, 160, 0, 0),
	CE(PALAU, FCC17_FCCA, 0, PW, 40, 160, 0, 0),
	CE(PANAMA, FCC14_FCCB, 0, PA, 40, 160, 0, 0),
	CE(PAPUA_NEW_GUINEA, FCC17_WORLD, 0, PG, 40, 160, 0, 0),
	CE(PARAGUAY, FCC17_WORLD, 0, PY, 40, 160, 0, 0),
	CE(PERU, FCC17_WORLD, FCC1_6G_05, PE, 40, 160, 320, 0),
	CE(PHILIPPINES, ETSI9_WORLD, 0, PH, 40, 160, 0, 0),
	CE(POLAND, ETSI10_WORLD,  ETSI1_6G_02, PL, 40, 160, 320, 0),
	CE(PORTUGAL, ETSI10_WORLD, ETSI1_6G_02, PT, 40, 160, 320, 0),
	CE(PUERTO_RICO, FCC10_FCCA, FCC1_6G_01, PR, 40, 160, 320, 0),
	CE(QATAR, ETSI14_WORLD, ETSI1_6G_02, QA, 40, 160, 320, 0),
	CE(REUNION, ETSI1_WORLD, ETSI1_6G_02, RE, 40, 160, 320, 0),
	CE(ROMANIA, ETSI10_WORLD, 0, RO, 40, 160, 0, 0),
	CE(RUSSIA, ETSI8_WORLD, 0, RU, 40, 160, 0, 0),
	CE(RWANDA, ETSI20_WORLD, 0, RW, 40, 160, 0, 0),
	CE(SAINT_BARTHELEMY, ETSI1_WORLD, 0, BL, 40, 160, 0, 0),
	CE(SAINT_HELENA_ASCENSION_AND_TRISTAN_DA_CUNHA, ETSI10_WORLD,
	   ETSI2_6G_03, SH, 40, 160, 320, 0),
	CE(SAINT_KITTS_AND_NEVIS, APL10_WORLD, 0, KN, 40, 160, 0, 0),
	CE(SAINT_LUCIA, APL10_WORLD, 0, LC, 40, 160, 0, 0),
	CE(SAINT_MARTIN, ETSI1_WORLD, 0, MF, 40, 160, 0, 0),
	CE(SAINT_PIERRE_AND_MIQUELON, ETSI13_WORLD, ETSI1_6G_02, PM, 40, 160,
	   320, 0),
	CE(SAINT_VINCENT_AND_THE_GRENADIENS, ETSI13_WORLD, 0, VC,
	   40, 160, 0, 0),
	CE(SAMOA, ETSI1_WORLD, 0, WS, 40, 40, 0, 0),
	CE(SAN_MARINO, ETSI1_WORLD,  ETSI1_6G_02, SM, 40, 160, 320, 0),
	CE(SAO_TOME_AND_PRINCIPE, FCC17_WORLD, 0, ST, 40, 160, 0, 0),
	CE(SAUDI_ARABIA, ETSI20_WORLD, APL6_6G_0A, SA, 40, 160, 320, 0),
	CE(SENEGAL, ETSI1_WORLD, 0, SN, 40, 160, 0, 0),
	CE(SERBIA, ETSI20_WORLD, 0, RS, 40, 160, 0, 0),
	CE(SINGAPORE, APL20_WORLD, 0, SG, 40, 160, 0, 0),
	CE(SINT_MAARTEN, ETSI10_WORLD, ETSI1_6G_02, SX, 40, 160, 320, 0),
	CE(SLOVAKIA, ETSI10_WORLD,  ETSI1_6G_02, SK, 40, 160, 320, 0),
	CE(SLOVENIA, ETSI10_WORLD,  ETSI1_6G_02, SI, 40, 160, 320, 0),
	CE(SOUTH_AFRICA, ETSI20_WORLD, 0, ZA, 40, 160, 0, 0),
	CE(SPAIN, ETSI10_WORLD, ETSI1_6G_02, ES, 40, 160, 320, 0),
	CE(SRI_LANKA, FCC17_ETSIC,  ETSI1_6G_02, LK, 40, 160, 320, 0),
	CE(SURINAME, ETSI1_WORLD, 0, SR, 40, 160, 0, 0),
	CE(SVALBARD_AND_JAN_MAYEN, ETSI13_WORLD, ETSI1_6G_02, SJ, 40, 160, 320,
	   0),
	CE(SWEDEN, ETSI10_WORLD, ETSI1_6G_02, SE, 40, 160, 320, 0),
	CE(SWITZERLAND, ETSI10_WORLD, ETSI1_6G_02, CH, 40, 160, 320, 0),
	CE(TAIWAN, FCC17_FCCA, 0, TW, 40, 160, 0, 0),
	CE(TANZANIA, APL1_WORLD, 0, TZ, 40, 160, 0, 0),
	CE(THAILAND, APL20_WORLD, 0, TH, 40, 160, 0, 0),
	CE(TOGO, ETSI1_WORLD, 0, TG, 40, 40, 0, 0),
	CE(TRINIDAD_Y_TOBAGO, FCC17_WORLD, 0, TT, 40, 160, 0, 0),
	CE(TUNISIA, ETSI3_WORLD, 0, TN, 40, 160, 0, 0),
	CE(TURKEY, ETSI20_WORLD,  ETSI1_6G_02, TR, 40, 160, 320, 0),
	CE(TURKS_AND_CAICOS, FCC17_WORLD, 0, TC, 40, 160, 0, 0),
	CE(UGANDA, FCC17_WORLD, 0, UG, 40, 160, 0, 0),
	CE(UKRAINE, ETSI9_WORLD, 0, UA, 40, 160, 0, 0),
	CE(UAE, ETSI20_WORLD, APL3_6G_08, AE, 40, 160, 320, 0),
	CE(UNITED_KINGDOM, ETSI10_WORLD, ETSI2_6G_03, GB, 40, 160, 320, 0),
	CE(UNITED_STATES, FCC8_FCCA, FCC1_6G_01, US, 40, 160, 320, 0),
	CE(UNITED_STATES_MINOR_OUTLYING_ISLANDS, FCC10_FCCA, FCC1_6G_01, UM,
	   40, 160, 320, 0),
	CE(URUGUAY, FCC2_WORLD, 0, UY, 40, 160, 0, 0),
	CE(UZBEKISTAN, ETSI3_WORLD, 0, UZ, 40, 160, 0, 0),
	CE(VANUATU, FCC17_WORLD, 0, VU, 40, 160, 0, 0),
	CE(VENEZUELA, FCC2_ETSIC, FCC1_6G_01, VE, 40, 160, 320, 0),
	CE(VIET_NAM, ETSI15_WORLD, 0, VN, 40, 160, 0, 0),
	CE(VIRGIN_ISLANDS, FCC10_FCCA, FCC1_6G_01, VI, 40, 160, 320, 0),
	CE(VIRGIN_ISLANDS_BRITISH, ETSI10_WORLD, ETSI2_6G_03, VG, 40, 160, 320,
	   0),
	CE(WALLIS_AND_FUTUNA, ETSI1_WORLD, 0, WF, 40, 160, 0, 0),
	CE(YEMEN, ETSI1_WORLD, 0, YE, 40, 160, 0, 0),
	CE(ZAMBIA, ETSI20_WORLD, 0, ZM, 40, 160, 0, 0),
	CE(ZIMBABWE, ETSI1_WORLD, 0, ZW, 40, 160, 0, 0),
};
#elif defined(WLAN_REG_AUTO)
const struct country_code_to_reg_domain g_all_countries[] = {
	CE(AFGHANISTAN, ETSI1_WORLD, 0, AF, 40, 160, 0, 0),
	CE(ALAND_ISLANDS, ETSI21_WORLD, ETSI1_6G_02, AX, 40, 160, 320, 0),
	CE(ALBANIA, ETSI20_WORLD, 0, AL, 40, 160, 0, 0),
	CE(ALGERIA, APL13_WORLD, 0, DZ, 40, 160, 0, 0),
	CE(AMERICAN_SAMOA, FCC3_FCCA, FCC1_6G_01, AS, 40, 160, 320, 0),
	CE(ANDORRA, ETSI21_WORLD, ETSI1_6G_02, AD, 40, 160, 320, 0),
	CE(ANGUILLA, ETSI1_WORLD, 0, AI, 40, 160, 0, 0),
	CE(ANTIGUA_AND_BARBUDA, FCC17_WORLD, FCC1_6G_01, AG, 40, 160, 320, 0),
	CE(ARGENTINA, APL17_ETSIC, 0, AR, 40, 160, 0, 0),
	CE(ARMENIA, APL4_WORLD, 0, AM, 40, 160, 0, 0),
	CE(ARUBA, ETSI1_WORLD, FCC1_6G_01, AW, 40, 160, 320, 0),
	CE(AUSTRALIA, APL25_ETSIC, ETSI2_6G_0C, AU, 40, 160, 320, 0),
	CE(AUSTRIA, ETSI21_WORLD, 0, AT, 40, 160, 0, 0),
	CE(AZERBAIJAN, ETSI8_WORLD, 0, AZ, 40, 160, 0, 0),
	CE(BAHAMAS, FCC17_WORLD, FCC1_6G_01, BS, 40, 160, 320, 0),
	CE(BAHRAIN, APL15_WORLD, 0, BH, 40, 160, 0, 0),
	CE(BANGLADESH, APL1_WORLD, 0, BD, 40, 160, 0, 0),
	CE(BARBADOS, FCC2_WORLD, 0, BB, 40, 160, 0, 0),
	CE(BELARUS, ETSI1_WORLD, 0, BY, 40, 160, 0, 0),
	CE(BELGIUM, ETSI21_WORLD, ETSI1_6G_02, BE, 40, 160, 320, 0),
	CE(BELIZE, ETSI8_WORLD, 0, BZ, 40, 160, 0, 0),
	CE(BERMUDA, FCC17_FCCA, 0, BM, 40, 160, 0, 0),
	CE(BHUTAN, ETSI1_WORLD, 0, BT, 40, 160, 0, 0),
	CE(BOLIVIA, FCC17_WORLD, 0, BO, 40, 160, 0, 0),
	CE(BOSNIA_HERZ, ETSI20_WORLD, 0, BA, 40, 160, 0, 0),
	CE(BOTSWANA, ETSI20_WORLD, 0, BW, 40, 160, 0, 0),
	CE(BRAZIL, FCC21_ETSIC, FCC1_6G_07, BR, 40, 160, 320, 0),
	CE(BRUNEI_DARUSSALAM, FCC17_WORLD, 0, BN, 40, 160, 0, 0),
	CE(BULGARIA, ETSI21_WORLD, ETSI1_6G_02, BG, 40, 160, 320, 0),
	CE(BURKINA_FASO, FCC17_WORLD, 0, BF, 40, 160, 0, 0),
	CE(BURUNDI, ETSI20_WORLD, 0, BI, 40, 160, 0, 0),
	CE(CAMBODIA, ETSI1_WORLD, 0, KH, 40, 160, 0, 0),
	CE(CAMEROON, ETSI1_WORLD, 0, CM, 40, 160, 0, 0),
	CE(CANADA, FCC6_FCCA, FCC1_6G_05, CA, 40, 160, 320, 0),
	CE(CAYMAN_ISLANDS, FCC17_WORLD, FCC1_6G_01, KY, 40, 160, 320, 0),
	CE(CENTRAL_AFRICA_REPUBLIC, FCC17_WORLD, 0, CF, 40, 40, 0, 0),
	CE(CHAD, ETSI1_WORLD, 0, TD, 40, 160, 0, 0),
	CE(CHILE, FCC20_FCCA, APL7_6G_13, CL, 40, 160, 320, 0),
	CE(CHINA, APL14_CHNA, 0, CN, 40, 160, 0, 0),
	CE(CHRISTMAS_ISLAND, APL25_ETSIC, ETSI2_6G_0C, CX, 40, 160, 320, 0),
	CE(COLOMBIA, FCC17_FCCA, 0, CO, 40, 160, 0, 0),
	CE(CONGO, ETSI12_WORLD, 0, CG, 40, 160, 0, 0),
	CE(CONGO_DEMOCRATIC_REPUBLIC, ETSI20_WORLD, 0, CD, 40, 160, 0, 0),
	CE(COOK_ISLANDS, APL26_ETSIC, ETSI2_6G_0E, CK, 40, 160, 320, 0),
	CE(COSTA_RICA, FCC17_WORLD, APL4_6G_11, CR, 40, 160, 320, 0),
	CE(COTE_DIVOIRE, FCC17_WORLD, 0, CI, 40, 160, 0, 0),
	CE(CROATIA, ETSI21_WORLD, 0, HR, 40, 160, 0, 0),
	CE(CURACAO, APL10_WORLD, ETSI1_6G_02, CW, 40, 160, 320, 0),
	CE(CYPRUS, ETSI21_WORLD, ETSI1_6G_02, CY, 40, 160, 320, 0),
	CE(CZECH, ETSI21_WORLD, ETSI1_6G_02, CZ, 40, 160, 320, 0),
	CE(DENMARK, ETSI21_WORLD, ETSI1_6G_02, DK, 40, 160, 320, 0),
	CE(DOMINICA, FCC2_FCCA, 0, DM, 40, 160, 0, 0),
	CE(DOMINICAN_REPUBLIC, FCC17_FCCA, FCC1_6G_14, DO, 40, 160, 320, 0),
	CE(ECUADOR, FCC17_FCCA, 0, EC, 40, 160, 0, 0),
	CE(EGYPT, ETSI3_WORLD, 0, EG, 40, 160, 0, 0),
	CE(EL_SALVADOR, FCC17_WORLD, FCC1_6G_01, SV, 40, 160, 320, 0),
	CE(ESTONIA, ETSI21_WORLD, ETSI1_6G_02, EE, 40, 160, 320, 0),
	CE(ETHIOPIA, ETSI1_WORLD, 0, ET, 40, 160, 0, 0),
	CE(FALKLAND_ISLANDS, ETSI21_WORLD, ETSI2_6G_03, FK, 40, 160, 320, 0),
	CE(FAROE_ISLANDS, ETSI21_WORLD, ETSI1_6G_02, FO, 40, 160, 320, 0),
	CE(FIJI, APL1_WORLD, ETSI2_6G_0C, FJ, 40, 160, 320, 0),
	CE(FINLAND, ETSI21_WORLD, ETSI1_6G_02, FI, 40, 160, 320, 0),
	CE(FRANCE, ETSI21_WORLD, ETSI1_6G_02, FR, 40, 160, 320, 0),
	CE(FRENCH_GUIANA, ETSI21_WORLD, ETSI1_6G_02, GF, 40, 160, 320, 0),
	CE(FRENCH_POLYNESIA, ETSI21_WORLD, ETSI1_6G_02, PF, 40, 160, 320, 0),
	CE(FRENCH_SOUTHERN_TERRITORIES, ETSI21_WORLD, ETSI1_6G_02, TF, 40, 160,
	   320, 0),
	CE(GABON, ETSI3_WORLD, 0, GA, 40, 160, 0, 0),
	CE(GEORGIA, ETSI1_WORLD, ETSI1_6G_02, GE, 40, 160, 320, 0),
	CE(GERMANY, ETSI21_WORLD, ETSI1_6G_02, DE, 40, 160, 320, 0),
	CE(GHANA, FCC17_WORLD, 0, GH, 40, 160, 0, 0),
	CE(GIBRALTAR, ETSI1_WORLD, ETSI2_6G_03, GI, 40, 160, 320, 0),
	CE(GREECE, ETSI21_WORLD,  ETSI1_6G_02, GR, 40, 160, 320, 0),
	CE(GREENLAND, ETSI1_WORLD, 0, GL, 40, 160, 0, 0),
	CE(GRENADA, FCC17_FCCA, FCC1_6G_01, GD, 40, 160, 320, 0),
	CE(GUADELOUPE, ETSI1_WORLD, ETSI1_6G_02, GP, 40, 160, 320, 0),
	CE(GUAM, FCC3_FCCA, FCC1_6G_01, GU, 40, 160, 320, 0),
	CE(GUATEMALA, APL29_ETSIC, APL2_6G_06, GT, 40, 160, 320, 0),
	CE(GUERNSEY, ETSI21_WORLD, ETSI2_6G_03, GG, 40, 160, 320, 0),
	CE(GUYANA, APL1_ETSIC, 0, GY, 40, 160, 0, 0),
	CE(HAITI, FCC17_FCCA, 0, HT, 40, 160, 0, 0),
	CE(HEARD_ISLAND_AND_MCDONALD_ISLANDS, APL25_ETSIC, ETSI2_6G_0C, HM, 40,
	   160, 320, 0),
	CE(HOLY_SEE, ETSI21_WORLD,  ETSI1_6G_02, VA, 40, 160, 320, 0),
	CE(HONDURAS, FCC13_WORLD, FCC1_6G_07, HN, 40, 160, 320, 0),
	CE(HONG_KONG, APL20_WORLD, ETSI2_6G_03, HK, 40, 160, 320, 0),
	CE(HUNGARY, ETSI21_WORLD, ETSI1_6G_02, HU, 40, 160, 320, 0),
	CE(ICELAND, ETSI21_WORLD, ETSI1_6G_02, IS, 40, 160, 320, 0),
	CE(INDIA, APL19_ETSIC, 0, IN, 40, 160, 0, 0),
	CE(INDONESIA, APL2_ETSID, 0, ID, 40, 80, 0, 0),
	CE(IRAQ, ETSI1_WORLD, 0, IQ, 40, 160, 0, 0),
	CE(IRELAND, ETSI21_WORLD, ETSI1_6G_02, IE, 40, 160, 320, 0),
	CE(ISLE_OF_MAN, ETSI21_WORLD, ETSI2_6G_03, IM, 40, 160, 320, 0),
	CE(ISRAEL, ETSI19_WORLD, ETSI1_6G_0D, IL, 40, 160, 320, 0),
	CE(ITALY, ETSI21_WORLD,  ETSI1_6G_02, IT, 40, 160, 320, 0),
	CE(JAMAICA, FCC13_WORLD, FCC1_6G_01, JM, 40, 160, 320, 0),
	CE(JAPAN, MKK18_MKKC, MKK1_6G_0B, JP, 40, 160, 320, 0),
	CE(JAPAN15, MKK5_MKKC, 0, JP, 40, 160, 0, 0),
	CE(XA, MKK5_MKKA, 0, XA, 40, 160, 0, 0),
	CE(JERSEY, ETSI21_WORLD, ETSI2_6G_03, JE, 40, 160, 320, 0),
	CE(JORDAN, APL4_WORLD, ETSI1_6G_02, JO, 40, 160, 320, 0),
	CE(KAZAKHSTAN, MKK5_MKKC, 0, KZ, 40, 160, 0, 0),
	CE(KENYA, ETSI20_WORLD, ETSI1_6G_02, KE, 40, 160, 320, 0),
	CE(KOREA_ROC, APL9_KRRA, APL1_6G_04, KR, 40, 160, 160, 0),
	CE(KUWAIT, ETSI13_WORLD, ETSI1_6G_0D, KW, 40, 160, 320, 0),
	CE(LAO_PEOPLES_DEMOCRATIC_REPUBLIC, ETSI20_WORLD, FCC1_6G_05, LA, 40,
	   160, 320, 0),
	CE(LATVIA, ETSI21_WORLD,  ETSI1_6G_02, LV, 40, 160, 320, 0),
	CE(LEBANON, FCC17_WORLD, 0, LB, 40, 160, 0, 0),
	CE(LESOTHO, ETSI1_WORLD, 0, LS, 40, 160, 0, 0),
	CE(LIBYA, ETSI12_WORLD, 0, LY, 40, 160, 0, 0),
	CE(LIECHTENSTEIN, ETSI21_WORLD, ETSI1_6G_02, LI, 40, 160, 320, 0),
	CE(LITHUANIA, ETSI21_WORLD, ETSI1_6G_02, LT, 40, 160, 320, 0),
	CE(LUXEMBOURG, ETSI21_WORLD,  ETSI1_6G_02, LU, 40, 160, 320, 0),
	CE(MACAU, FCC17_WORLD, 0, MO, 40, 160, 0, 0),
	CE(MACEDONIA, ETSI20_WORLD, 0, MK, 40, 160, 0, 0),
	CE(MALAWI, ETSI1_WORLD, 0, MW, 40, 160, 0, 0),
	CE(MALAYSIA, FCC11_WORLD, ETSI1_6G_02, MY, 40, 160, 320, 0),
	CE(MALDIVES, APL6_WORLD, 0, MV, 40, 160, 0, 0),
	CE(MALTA, ETSI21_WORLD, ETSI1_6G_02, MT, 40, 160, 320, 0),
	CE(MARSHALL_ISLANDS, FCC17_FCCA, 0, MH, 40, 160, 0, 0),
	CE(MARTINIQUE, ETSI21_WORLD, ETSI1_6G_02, MQ, 40, 160, 320, 0),
	CE(MAURITANIA, ETSI1_WORLD, 0, MR, 40, 160, 0, 0),
	CE(MAURITIUS, ETSI20_WORLD, ETSI1_6G_02, MU, 40, 160, 320, 0),
	CE(MAYOTTE, ETSI21_WORLD, ETSI1_6G_02, YT, 40, 160, 320, 0),
	CE(MEXICO, APL16_ETSIC, 0, MX, 40, 160, 0, 0),
	CE(MICRONESIA, FCC17_FCCA, 0, FM, 40, 160, 0, 0),
	CE(MOLDOVA, ETSI20_WORLD, 0, MD, 40, 160, 0, 0),
	CE(MONACO, ETSI20_WORLD, 0, MC, 40, 160, 0, 0),
	CE(MONGOLIA, FCC17_WORLD, ETSI1_6G_02, MN, 40, 160, 320, 0),
	CE(MONTENEGRO, ETSI20_WORLD, 0, ME, 40, 160, 0, 0),
	CE(MONTSERRAT, ETSI21_WORLD, ETSI2_6G_03, MS, 40, 160, 320, 0),
	CE(MOROCCO, ETSI3_WORLD, ETSI1_6G_02, MA, 40, 160, 320, 0),
	CE(MYANMAR, APL1_WORLD, 0, MM, 40, 160, 0, 0),
	CE(NAMIBIA, APL20_WORLD, 0, NA, 40, 160, 0, 0),
	CE(NEPAL, APL23_WORLD, 0, NP, 40, 160, 0, 0),
	CE(NETHERLANDS, ETSI21_WORLD, ETSI1_6G_02, NL, 40, 160, 320, 0),
	CE(NETHERLANDS_ANTILLES, ETSI21_WORLD, ETSI1_6G_02, AN, 40, 160, 320,
	   0),
	CE(NEW_CALEDONIA, ETSI21_WORLD, ETSI1_6G_02, NC, 40, 160, 320, 0),
	CE(NEW_ZEALAND, APL26_ETSIC, ETSI2_6G_0E, NZ, 40, 160, 320, 0),
	CE(NICARAGUA, APL27_FCCA, 0, NI, 40, 160, 0, 0),
	CE(NIGERIA, APL6_WORLD, 0, NG, 40, 160, 0, 0),
	CE(NIUE, APL26_ETSIC, ETSI2_6G_0E, NU, 40, 160, 320, 0),
	CE(NORFOLK_ISLAND, APL25_ETSIC, ETSI2_6G_0C, NF, 40, 160, 320, 0),
	CE(NORTHERN_MARIANA_ISLANDS, FCC3_FCCA, FCC1_6G_01, MP, 40, 160, 320,
	   0),
	CE(NORWAY, ETSI21_WORLD, ETSI1_6G_02, NO, 40, 160, 320, 0),
	CE(OMAN, ETSI1_WORLD, 0, OM, 40, 160, 0, 0),
	CE(PAKISTAN, APL28_ETSIC, 0, PK, 40, 160, 0, 0),
	CE(PALAU, FCC17_FCCA, 0, PW, 40, 160, 0, 0),
	CE(PANAMA, FCC14_FCCB, 0, PA, 40, 160, 0, 0),
	CE(PAPUA_NEW_GUINEA, FCC17_WORLD, 0, PG, 40, 160, 0, 0),
	CE(PARAGUAY, FCC17_WORLD, 0, PY, 40, 160, 0, 0),
	CE(PERU, FCC17_WORLD, FCC1_6G_05, PE, 40, 160, 320, 0),
	CE(PHILIPPINES, ETSI9_WORLD, 0, PH, 40, 160, 0, 0),
	CE(POLAND, ETSI21_WORLD,  ETSI1_6G_02, PL, 40, 160, 320, 0),
	CE(PORTUGAL, ETSI21_WORLD, ETSI1_6G_02, PT, 40, 160, 320, 0),
	CE(PUERTO_RICO, FCC3_FCCA, FCC1_6G_01, PR, 40, 160, 320, 0),
	CE(QATAR, ETSI14_WORLD, ETSI1_6G_02, QA, 40, 160, 320, 0),
	CE(REUNION, ETSI21_WORLD, ETSI1_6G_02, RE, 40, 160, 320, 0),
	CE(ROMANIA, ETSI21_WORLD, 0, RO, 40, 160, 0, 0),
	CE(RUSSIA, ETSI8_WORLD, 0, RU, 40, 160, 0, 0),
	CE(RWANDA, ETSI20_WORLD, 0, RW, 40, 160, 0, 0),
	CE(SAINT_BARTHELEMY, ETSI1_WORLD, ETSI1_6G_02, BL, 40, 160, 0, 0),
	CE(SAINT_HELENA_ASCENSION_AND_TRISTAN_DA_CUNHA, ETSI21_WORLD,
	   ETSI2_6G_03, SH, 40, 160, 320, 0),
	CE(SAINT_KITTS_AND_NEVIS, APL10_WORLD, 0, KN, 40, 160, 0, 0),
	CE(SAINT_LUCIA, APL10_WORLD, 0, LC, 40, 160, 0, 0),
	CE(SAINT_MARTIN, ETSI1_WORLD, 0, MF, 40, 160, 0, 0),
	CE(SAINT_PIERRE_AND_MIQUELON, ETSI21_WORLD, ETSI1_6G_02, PM, 40, 160,
	   320, 0),
	CE(SAINT_VINCENT_AND_THE_GRENADIENS, ETSI13_WORLD, 0, VC,
	   40, 160, 0, 0),
	CE(SAMOA, ETSI1_WORLD, 0, WS, 40, 40, 0, 0),
	CE(SAN_MARINO, ETSI21_WORLD,  ETSI1_6G_02, SM, 40, 160, 320, 0),
	CE(SAO_TOME_AND_PRINCIPE, FCC17_WORLD, 0, ST, 40, 160, 0, 0),
	CE(SAUDI_ARABIA, ETSI20_WORLD, APL6_6G_0A, SA, 40, 160, 320, 0),
	CE(SENEGAL, ETSI1_WORLD, 0, SN, 40, 160, 0, 0),
	CE(SERBIA, ETSI20_WORLD, 0, RS, 40, 160, 0, 0),
	CE(SINGAPORE, APL20_WORLD, 0, SG, 40, 160, 0, 0),
	CE(SINT_MAARTEN, ETSI21_WORLD, ETSI1_6G_02, SX, 40, 160, 320, 0),
	CE(SLOVAKIA, ETSI21_WORLD,  ETSI1_6G_02, SK, 40, 160, 320, 0),
	CE(SLOVENIA, ETSI21_WORLD,  ETSI1_6G_02, SI, 40, 160, 320, 0),
	CE(SOUTH_AFRICA, ETSI20_WORLD, 0, ZA, 40, 160, 0, 0),
	CE(SPAIN, ETSI21_WORLD, ETSI1_6G_02, ES, 40, 160, 320, 0),
	CE(SRI_LANKA, ETSI15_ETSIC,  ETSI1_6G_02, LK, 40, 160, 320, 0),
	CE(SURINAME, ETSI1_WORLD, 0, SR, 40, 160, 0, 0),
	CE(SVALBARD_AND_JAN_MAYEN, ETSI21_WORLD, ETSI1_6G_02, SJ, 40, 160, 320,
	   0),
	CE(SWEDEN, ETSI21_WORLD, ETSI1_6G_02, SE, 40, 160, 320, 0),
	CE(SWITZERLAND, ETSI21_WORLD, ETSI1_6G_02, CH, 40, 160, 320, 0),
	CE(TAIWAN, FCC17_FCCA, 0, TW, 40, 160, 0, 0),
	CE(TANZANIA, ETSI15_WORLD, 0, TZ, 40, 160, 0, 0),
	CE(THAILAND, APL20_WORLD, 0, TH, 40, 160, 0, 0),
	CE(TOGO, ETSI1_WORLD, 0, TG, 40, 40, 0, 0),
	CE(TRINIDAD_Y_TOBAGO, FCC17_WORLD, 0, TT, 40, 160, 0, 0),
	CE(TUNISIA, ETSI3_WORLD, 0, TN, 40, 160, 0, 0),
	CE(TURKEY, ETSI20_WORLD,  ETSI1_6G_02, TR, 40, 160, 320, 0),
	CE(TURKS_AND_CAICOS, FCC17_WORLD, 0, TC, 40, 160, 0, 0),
	CE(UGANDA, FCC17_WORLD, 0, UG, 40, 160, 0, 0),
	CE(UKRAINE, ETSI9_WORLD, 0, UA, 40, 160, 0, 0),
	CE(UAE, ETSI20_WORLD, APL3_6G_08, AE, 40, 160, 320, 0),
	CE(UNITED_KINGDOM, ETSI21_WORLD, ETSI2_6G_03, GB, 40, 160, 320, 0),
	CE(UNITED_STATES, FCC3_FCCA, FCC1_6G_01, US, 40, 160, 320, 0),
	CE(UNITED_STATES_MINOR_OUTLYING_ISLANDS, FCC3_FCCA, FCC1_6G_01, UM, 40,
	   160, 320, 0),
	CE(URUGUAY, FCC2_WORLD, 0, UY, 40, 160, 0, 0),
	CE(UZBEKISTAN, ETSI3_WORLD, 0, UZ, 40, 160, 0, 0),
	CE(VANUATU, FCC17_WORLD, 0, VU, 40, 160, 0, 0),
	CE(VENEZUELA, FCC2_ETSIC, FCC1_6G_01, VE, 40, 160, 320, 0),
	CE(VIET_NAM, ETSI15_WORLD, 0, VN, 40, 160, 0, 0),
	CE(VIRGIN_ISLANDS, FCC3_FCCA, FCC1_6G_01, VI, 40, 160, 320, 0),
	CE(VIRGIN_ISLANDS_BRITISH, ETSI21_WORLD, ETSI2_6G_03, VG, 40, 160, 320,
	   0),
	CE(WALLIS_AND_FUTUNA, ETSI1_WORLD, 0, WF, 40, 160, 0, 0),
	CE(YEMEN, ETSI1_WORLD, 0, YE, 40, 160, 0, 0),
	CE(ZAMBIA, ETSI20_WORLD, 0, ZM, 40, 160, 0, 0),
	CE(ZIMBABWE, ETSI1_WORLD, 0, ZW, 40, 160, 0, 0),
};
#else
const struct country_code_to_reg_domain g_all_countries[] = {
	CE(AFGHANISTAN, ETSI1_WORLD, 0, AF, 40, 160, 0, 0),
	CE(ALAND_ISLANDS, ETSI13_WORLD, ETSI1_6G_02, AX, 40, 160, 320, 0),
	CE(ALBANIA, ETSI20_WORLD, 0, AL, 40, 160, 0, 0),
	CE(ALGERIA, APL13_WORLD, 0, DZ, 40, 160, 0, 0),
	CE(AMERICAN_SAMOA, FCC3_FCCA, FCC1_6G_01, AS, 40, 160, 320, 0),
	CE(ANDORRA, ETSI13_WORLD, ETSI1_6G_02, AD, 40, 160, 320, 0),
	CE(ANGUILLA, ETSI1_WORLD, 0, AI, 40, 160, 0, 0),
	CE(ANTIGUA_AND_BARBUDA, FCC17_WORLD, FCC1_6G_01, AG, 40, 160, 320, 0),
	CE(ARGENTINA, APL17_ETSIC, 0, AR, 40, 160, 0, 0),
	CE(ARMENIA, APL4_WORLD, 0, AM, 40, 160, 0, 0),
	CE(ARUBA, ETSI1_WORLD, FCC1_6G_01, AW, 40, 160, 320, 0),
	CE(AUSTRALIA, APL25_ETSIC, ETSI2_6G_0C, AU, 40, 160, 320, 0),
	CE(AUSTRIA, ETSI13_WORLD, 0, AT, 40, 160, 0, 0),
	CE(AZERBAIJAN, ETSI8_WORLD, 0, AZ, 40, 160, 0, 0),
	CE(BAHAMAS, FCC17_WORLD, FCC1_6G_01, BS, 40, 160, 320, 0),
	CE(BAHRAIN, APL15_WORLD, 0, BH, 40, 160, 0, 0),
	CE(BANGLADESH, APL1_WORLD, 0, BD, 40, 160, 0, 0),
	CE(BARBADOS, FCC2_WORLD, 0, BB, 40, 160, 0, 0),
	CE(BELARUS, ETSI1_WORLD, 0, BY, 40, 160, 0, 0),
	CE(BELGIUM, ETSI13_WORLD, ETSI1_6G_02, BE, 40, 160, 320, 0),
	CE(BELIZE, ETSI8_WORLD, 0, BZ, 40, 160, 0, 0),
	CE(BERMUDA, FCC17_FCCA, 0, BM, 40, 160, 0, 0),
	CE(BHUTAN, ETSI1_WORLD, 0, BT, 40, 160, 0, 0),
	CE(BOLIVIA, FCC17_WORLD, 0, BO, 40, 160, 0, 0),
	CE(BOSNIA_HERZ, ETSI20_WORLD, 0, BA, 40, 160, 0, 0),
	CE(BOTSWANA, ETSI20_WORLD, 0, BW, 40, 160, 0, 0),
	CE(BRAZIL, FCC18_ETSIC, FCC1_6G_07, BR, 40, 160, 320, 0),
	CE(BRUNEI_DARUSSALAM, FCC17_WORLD, 0, BN, 40, 160, 0, 0),
	CE(BULGARIA, ETSI13_WORLD, ETSI1_6G_02, BG, 40, 160, 320, 0),
	CE(BURKINA_FASO, FCC17_WORLD, 0, BF, 40, 160, 0, 0),
	CE(BURUNDI, ETSI20_WORLD, 0, BI, 40, 160, 0, 0),
	CE(CAMBODIA, ETSI1_WORLD, 0, KH, 40, 160, 0, 0),
	CE(CAMEROON, ETSI1_WORLD, 0, CM, 40, 160, 0, 0),
	CE(CANADA, FCC6_FCCA, FCC1_6G_05, CA, 40, 160, 320, 0),
	CE(CAYMAN_ISLANDS, FCC17_WORLD, FCC1_6G_01, KY, 40, 160, 320, 0),
	CE(CENTRAL_AFRICA_REPUBLIC, FCC17_WORLD, 0, CF, 40, 40, 0, 0),
	CE(CHAD, ETSI1_WORLD, 0, TD, 40, 160, 0, 0),
	CE(CHILE, FCC20_FCCA, APL7_6G_13, CL, 40, 160, 320, 0),
	CE(CHINA, APL14_CHNA, 0, CN, 40, 160, 0, 0),
	CE(CHRISTMAS_ISLAND, APL25_ETSIC, ETSI2_6G_0C, CX, 40, 160, 320, 0),
	CE(COLOMBIA, FCC17_FCCA, 0, CO, 40, 160, 0, 0),
	CE(CONGO, ETSI12_WORLD, 0, CG, 40, 160, 0, 0),
	CE(CONGO_DEMOCRATIC_REPUBLIC, ETSI20_WORLD, 0, CD, 40, 160, 0, 0),
	CE(COOK_ISLANDS, APL26_ETSIC, ETSI2_6G_0E, CK, 40, 160, 320, 0),
	CE(COSTA_RICA, FCC17_WORLD, APL4_6G_11, CR, 40, 160, 320, 0),
	CE(COTE_DIVOIRE, FCC17_WORLD, 0, CI, 40, 160, 0, 0),
	CE(CROATIA, ETSI13_WORLD, 0, HR, 40, 160, 0, 0),
	CE(CURACAO, APL10_WORLD, ETSI1_6G_02, CW, 40, 160, 320, 0),
	CE(CYPRUS, ETSI13_WORLD, ETSI1_6G_02, CY, 40, 160, 320, 0),
	CE(CZECH, ETSI13_WORLD, ETSI1_6G_02, CZ, 40, 160, 320, 0),
	CE(DENMARK, ETSI13_WORLD, ETSI1_6G_02, DK, 40, 160, 320, 0),
	CE(DOMINICA, FCC2_FCCA, 0, DM, 40, 160, 0, 0),
	CE(DOMINICAN_REPUBLIC, FCC17_FCCA, FCC1_6G_14, DO, 40, 160, 320, 0),
	CE(ECUADOR, FCC17_FCCA, 0, EC, 40, 160, 0, 0),
	CE(EGYPT, ETSI3_WORLD, 0, EG, 40, 160, 0, 0),
	CE(EL_SALVADOR, FCC17_WORLD, FCC1_6G_01, SV, 40, 160, 320, 0),
	CE(ESTONIA, ETSI13_WORLD, ETSI1_6G_02, EE, 40, 160, 320, 0),
	CE(ETHIOPIA, ETSI1_WORLD, 0, ET, 40, 160, 0, 0),
	CE(FALKLAND_ISLANDS, ETSI18_WORLD, ETSI2_6G_03, FK, 40, 160, 320, 0),
	CE(FAROE_ISLANDS, ETSI13_WORLD, ETSI1_6G_02, FO, 40, 160, 320, 0),
	CE(FIJI, APL1_WORLD, ETSI2_6G_0C, FJ, 40, 160, 320, 0),
	CE(FINLAND, ETSI13_WORLD, ETSI1_6G_02, FI, 40, 160, 320, 0),
	CE(FRANCE, ETSI13_WORLD, ETSI1_6G_02, FR, 40, 160, 320, 0),
	CE(FRENCH_GUIANA, ETSI13_WORLD, ETSI1_6G_02, GF, 40, 160, 320, 0),
	CE(FRENCH_POLYNESIA, ETSI13_WORLD, ETSI1_6G_02, PF, 40, 160, 320, 0),
	CE(FRENCH_SOUTHERN_TERRITORIES, ETSI13_WORLD, ETSI1_6G_02, TF, 40, 160,
	   320, 0),
	CE(GABON, ETSI3_WORLD, 0, GA, 40, 160, 0, 0),
	CE(GEORGIA, ETSI1_WORLD, ETSI1_6G_02, GE, 40, 160, 320, 0),
	CE(GERMANY, ETSI13_WORLD, ETSI1_6G_02, DE, 40, 160, 320, 0),
	CE(GHANA, FCC17_WORLD, 0, GH, 40, 160, 0, 0),
	CE(GIBRALTAR, ETSI18_WORLD, ETSI2_6G_03, GI, 40, 160, 320, 0),
	CE(GREECE, ETSI13_WORLD,  ETSI1_6G_02, GR, 40, 160, 320, 0),
	CE(GREENLAND, ETSI1_WORLD, 0, GL, 40, 160, 0, 0),
	CE(GRENADA, FCC17_FCCA, FCC1_6G_01, GD, 40, 160, 320, 0),
	CE(GUADELOUPE, ETSI1_WORLD, ETSI1_6G_02, GP, 40, 160, 320, 0),
	CE(GUAM, FCC3_FCCA, FCC1_6G_01, GU, 40, 160, 320, 0),
	CE(GUATEMALA, APL29_ETSIC, APL2_6G_06, GT, 40, 160, 320, 0),
	CE(GUERNSEY, ETSI18_WORLD, ETSI2_6G_03, GG, 40, 160, 320, 0),
	CE(GUYANA, APL1_ETSIC, 0, GY, 40, 160, 0, 0),
	CE(HAITI, FCC17_FCCA, 0, HT, 40, 160, 0, 0),
	CE(HEARD_ISLAND_AND_MCDONALD_ISLANDS, APL25_ETSIC, ETSI2_6G_0C, HM, 40,
	   160, 320, 0),
	CE(HOLY_SEE, ETSI13_WORLD,  ETSI1_6G_02, VA, 40, 160, 320, 0),
	CE(HONDURAS, FCC13_WORLD, FCC1_6G_07, HN, 40, 160, 320, 0),
	CE(HONG_KONG, APL20_WORLD, ETSI2_6G_03, HK, 40, 160, 320, 0),
	CE(HUNGARY, ETSI13_WORLD, ETSI1_6G_02, HU, 40, 160, 320, 0),
	CE(ICELAND, ETSI13_WORLD, ETSI1_6G_02, IS, 40, 160, 320, 0),
	CE(INDIA, APL19_ETSIC, 0, IN, 40, 160, 0, 0),
	CE(INDONESIA, APL2_ETSID, 0, ID, 40, 80, 0, 0),
	CE(IRAQ, ETSI1_WORLD, 0, IQ, 40, 160, 0, 0),
	CE(IRELAND, ETSI13_WORLD, ETSI1_6G_02, IE, 40, 160, 320, 0),
	CE(ISLE_OF_MAN, ETSI18_WORLD, ETSI2_6G_03, IM, 40, 160, 320, 0),
	CE(ISRAEL, ETSI17_WORLD, ETSI1_6G_0D, IL, 40, 160, 320, 0),
	CE(ITALY, ETSI13_WORLD,  ETSI1_6G_02, IT, 40, 160, 320, 0),
	CE(JAMAICA, FCC13_WORLD, FCC1_6G_01, JM, 40, 160, 320, 0),
	CE(JAPAN, MKK17_MKKC, MKK1_6G_0B, JP, 40, 160, 320, 0),
	CE(JAPAN15, MKK5_MKKC, 0, JP, 40, 160, 0, 0),
	CE(XA, MKK5_MKKA, 0, XA, 40, 160, 0, 0),
	CE(JERSEY, ETSI18_WORLD, ETSI2_6G_03, JE, 40, 160, 320, 0),
	CE(JORDAN, APL4_WORLD, ETSI1_6G_02, JO, 40, 160, 320, 0),
	CE(KAZAKHSTAN, MKK5_MKKC, 0, KZ, 40, 160, 0, 0),
	CE(KENYA, ETSI20_WORLD, ETSI1_6G_02, KE, 40, 160, 320, 0),
	CE(KOREA_ROC, APL9_KRRA, APL1_6G_04, KR, 40, 160, 160, 0),
	CE(KUWAIT, ETSI13_WORLD, ETSI1_6G_0D, KW, 40, 160, 320, 0),
	CE(LAO_PEOPLES_DEMOCRATIC_REPUBLIC, ETSI20_WORLD, FCC1_6G_05, LA, 40,
	   160, 320, 0),
	CE(LATVIA, ETSI13_WORLD,  ETSI1_6G_02, LV, 40, 160, 320, 0),
	CE(LEBANON, FCC17_WORLD, 0, LB, 40, 160, 0, 0),
	CE(LESOTHO, ETSI1_WORLD, 0, LS, 40, 160, 0, 0),
	CE(LIBYA, ETSI12_WORLD, 0, LY, 40, 160, 0, 0),
	CE(LIECHTENSTEIN, ETSI13_WORLD, ETSI1_6G_02, LI, 40, 160, 320, 0),
	CE(LITHUANIA, ETSI13_WORLD, ETSI1_6G_02, LT, 40, 160, 320, 0),
	CE(LUXEMBOURG, ETSI13_WORLD,  ETSI1_6G_02, LU, 40, 160, 320, 0),
	CE(MACAU, FCC17_WORLD, 0, MO, 40, 160, 0, 0),
	CE(MACEDONIA, ETSI20_WORLD, 0, MK, 40, 160, 0, 0),
	CE(MALAWI, ETSI1_WORLD, 0, MW, 40, 160, 0, 0),
	CE(MALAYSIA, FCC11_WORLD, ETSI1_6G_02, MY, 40, 160, 320, 0),
	CE(MALDIVES, APL6_WORLD, 0, MV, 40, 160, 0, 0),
	CE(MALTA, ETSI13_WORLD, ETSI1_6G_02, MT, 40, 160, 320, 0),
	CE(MARSHALL_ISLANDS, FCC17_FCCA, 0, MH, 40, 160, 0, 0),
	CE(MARTINIQUE, ETSI13_WORLD, ETSI1_6G_02, MQ, 40, 160, 320, 0),
	CE(MAURITANIA, ETSI1_WORLD, 0, MR, 40, 160, 0, 0),
	CE(MAURITIUS, ETSI20_WORLD, ETSI1_6G_02, MU, 40, 160, 320, 0),
	CE(MAYOTTE, ETSI1_WORLD, ETSI1_6G_02, YT, 40, 160, 320, 0),
	CE(MEXICO, APL16_ETSIC, 0, MX, 40, 160, 0, 0),
	CE(MICRONESIA, FCC17_FCCA, 0, FM, 40, 160, 0, 0),
	CE(MOLDOVA, ETSI20_WORLD, 0, MD, 40, 160, 0, 0),
	CE(MONACO, ETSI20_WORLD, 0, MC, 40, 160, 0, 0),
	CE(MONGOLIA, FCC17_WORLD, ETSI1_6G_02, MN, 40, 160, 320, 0),
	CE(MONTENEGRO, ETSI20_WORLD, 0, ME, 40, 160, 0, 0),
	CE(MONTSERRAT, ETSI18_WORLD, ETSI2_6G_03, MS, 40, 160, 320, 0),
	CE(MOROCCO, ETSI3_WORLD, ETSI1_6G_02, MA, 40, 160, 320, 0),
	CE(MYANMAR, APL1_WORLD, 0, MM, 40, 160, 0, 0),
	CE(NAMIBIA, APL20_WORLD, 0, NA, 40, 160, 0, 0),
	CE(NEPAL, APL23_WORLD, 0, NP, 40, 160, 0, 0),
	CE(NETHERLANDS, ETSI13_WORLD, ETSI1_6G_02, NL, 40, 160, 320, 0),
	CE(NETHERLANDS_ANTILLES, ETSI13_WORLD, ETSI1_6G_02, AN, 40, 160, 320,
	   0),
	CE(NEW_CALEDONIA, ETSI13_WORLD, ETSI1_6G_02, NC, 40, 160, 320, 0),
	CE(NEW_ZEALAND, APL26_ETSIC, ETSI2_6G_0E, NZ, 40, 160, 320, 0),
	CE(NICARAGUA, APL27_FCCA, 0, NI, 40, 160, 0, 0),
	CE(NIGERIA, APL6_WORLD, 0, NG, 40, 160, 0, 0),
	CE(NIUE, APL26_ETSIC, ETSI2_6G_0E, NU, 40, 160, 320, 0),
	CE(NORFOLK_ISLAND, APL25_ETSIC, ETSI2_6G_0C, NF, 40, 160, 320, 0),
	CE(NORTHERN_MARIANA_ISLANDS, FCC3_FCCA, FCC1_6G_01, MP, 40, 160, 320,
	   0),
	CE(NORWAY, ETSI13_WORLD, ETSI1_6G_02, NO, 40, 160, 320, 0),
	CE(OMAN, ETSI1_WORLD, 0, OM, 40, 160, 0, 0),
	CE(PAKISTAN, APL28_ETSIC, 0, PK, 40, 160, 0, 0),
	CE(PALAU, FCC17_FCCA, 0, PW, 40, 160, 0, 0),
	CE(PANAMA, FCC14_FCCB, 0, PA, 40, 160, 0, 0),
	CE(PAPUA_NEW_GUINEA, FCC17_WORLD, 0, PG, 40, 160, 0, 0),
	CE(PARAGUAY, FCC17_WORLD, 0, PY, 40, 160, 0, 0),
	CE(PERU, FCC17_WORLD, FCC1_6G_05, PE, 40, 160, 320, 0),
	CE(PHILIPPINES, ETSI9_WORLD, 0, PH, 40, 160, 0, 0),
	CE(POLAND, ETSI13_WORLD,  ETSI1_6G_02, PL, 40, 160, 320, 0),
	CE(PORTUGAL, ETSI13_WORLD, ETSI1_6G_02, PT, 40, 160, 320, 0),
	CE(PUERTO_RICO, FCC3_FCCA, FCC1_6G_01, PR, 40, 160, 320, 0),
	CE(QATAR, ETSI14_WORLD, ETSI1_6G_02, QA, 40, 160, 320, 0),
	CE(REUNION, ETSI13_WORLD, ETSI1_6G_02, RE, 40, 160, 320, 0),
	CE(ROMANIA, ETSI13_WORLD, 0, RO, 40, 160, 0, 0),
	CE(RUSSIA, ETSI8_WORLD, 0, RU, 40, 160, 0, 0),
	CE(RWANDA, ETSI20_WORLD, 0, RW, 40, 160, 0, 0),
	CE(SAINT_BARTHELEMY, ETSI1_WORLD, ETSI1_6G_02, BL, 40, 160, 0, 0),
	CE(SAINT_HELENA_ASCENSION_AND_TRISTAN_DA_CUNHA, ETSI18_WORLD,
	   ETSI2_6G_03, SH, 40, 160, 320, 0),
	CE(SAINT_KITTS_AND_NEVIS, APL10_WORLD, 0, KN, 40, 160, 0, 0),
	CE(SAINT_LUCIA, APL10_WORLD, 0, LC, 40, 160, 0, 0),
	CE(SAINT_MARTIN, ETSI1_WORLD, 0, MF, 40, 160, 0, 0),
	CE(SAINT_PIERRE_AND_MIQUELON, ETSI13_WORLD, ETSI1_6G_02, PM, 40, 160,
	   320, 0),
	CE(SAINT_VINCENT_AND_THE_GRENADIENS, ETSI13_WORLD, 0, VC,
	   40, 160, 0, 0),
	CE(SAMOA, ETSI1_WORLD, 0, WS, 40, 40, 0, 0),
	CE(SAN_MARINO, ETSI13_WORLD,  ETSI1_6G_02, SM, 40, 160, 320, 0),
	CE(SAO_TOME_AND_PRINCIPE, FCC17_WORLD, 0, ST, 40, 160, 0, 0),
	CE(SAUDI_ARABIA, ETSI20_WORLD, APL6_6G_0A, SA, 40, 160, 320, 0),
	CE(SENEGAL, ETSI1_WORLD, 0, SN, 40, 160, 0, 0),
	CE(SERBIA, ETSI20_WORLD, 0, RS, 40, 160, 0, 0),
	CE(SINGAPORE, APL20_WORLD, 0, SG, 40, 160, 0, 0),
	CE(SINT_MAARTEN, ETSI13_WORLD, ETSI1_6G_02, SX, 40, 160, 320, 0),
	CE(SLOVAKIA, ETSI13_WORLD,  ETSI1_6G_02, SK, 40, 160, 320, 0),
	CE(SLOVENIA, ETSI13_WORLD,  ETSI1_6G_02, SI, 40, 160, 320, 0),
	CE(SOUTH_AFRICA, ETSI20_WORLD, 0, ZA, 40, 160, 0, 0),
	CE(SPAIN, ETSI13_WORLD, ETSI1_6G_02, ES, 40, 160, 320, 0),
	CE(SRI_LANKA, ETSI15_ETSIC,  ETSI1_6G_02, LK, 40, 160, 320, 0),
	CE(SURINAME, ETSI1_WORLD, 0, SR, 40, 160, 0, 0),
	CE(SVALBARD_AND_JAN_MAYEN, ETSI13_WORLD, ETSI1_6G_02, SJ, 40, 160, 320,
	   0),
	CE(SWEDEN, ETSI13_WORLD, ETSI1_6G_02, SE, 40, 160, 320, 0),
	CE(SWITZERLAND, ETSI13_WORLD, ETSI1_6G_02, CH, 40, 160, 320, 0),
	CE(TAIWAN, FCC17_FCCA, 0, TW, 40, 160, 0, 0),
	CE(TANZANIA, ETSI15_WORLD, 0, TZ, 40, 160, 0, 0),
	CE(THAILAND, APL20_WORLD, 0, TH, 40, 160, 0, 0),
	CE(TOGO, ETSI1_WORLD, 0, TG, 40, 40, 0, 0),
	CE(TRINIDAD_Y_TOBAGO, FCC17_WORLD, 0, TT, 40, 160, 0, 0),
	CE(TUNISIA, ETSI3_WORLD, 0, TN, 40, 160, 0, 0),
	CE(TURKEY, ETSI20_WORLD,  ETSI1_6G_02, TR, 40, 160, 320, 0),
	CE(TURKS_AND_CAICOS, FCC17_WORLD, 0, TC, 40, 160, 0, 0),
	CE(UGANDA, FCC17_WORLD, 0, UG, 40, 160, 0, 0),
	CE(UKRAINE, ETSI9_WORLD, 0, UA, 40, 160, 0, 0),
	CE(UAE, ETSI20_WORLD, APL3_6G_08, AE, 40, 160, 320, 0),
	CE(UNITED_KINGDOM, ETSI18_WORLD, ETSI2_6G_03, GB, 40, 160, 320, 0),
	CE(UNITED_STATES, FCC8_FCCA, FCC1_6G_01, US, 40, 160, 320, 0),
	CE(UNITED_STATES_MINOR_OUTLYING_ISLANDS, FCC3_FCCA, FCC1_6G_01, UM, 40,
	   160, 320, 0),
	CE(URUGUAY, FCC2_WORLD, 0, UY, 40, 160, 0, 0),
	CE(UZBEKISTAN, ETSI3_WORLD, 0, UZ, 40, 160, 0, 0),
	CE(VANUATU, FCC17_WORLD, 0, VU, 40, 160, 0, 0),
	CE(VENEZUELA, FCC2_ETSIC, FCC1_6G_01, VE, 40, 160, 320, 0),
	CE(VIET_NAM, ETSI15_WORLD, 0, VN, 40, 160, 0, 0),
	CE(VIRGIN_ISLANDS, FCC3_FCCA, FCC1_6G_01, VI, 40, 160, 320, 0),
	CE(VIRGIN_ISLANDS_BRITISH, ETSI18_WORLD, ETSI2_6G_03, VG, 40, 160, 320,
	   0),
	CE(WALLIS_AND_FUTUNA, ETSI1_WORLD, 0, WF, 40, 160, 0, 0),
	CE(YEMEN, ETSI1_WORLD, 0, YE, 40, 160, 0, 0),
	CE(ZAMBIA, ETSI20_WORLD, 0, ZM, 40, 160, 0, 0),
	CE(ZIMBABWE, ETSI1_WORLD, 0, ZW, 40, 160, 0, 0),
};
#endif
#endif
const struct reg_domain_pair g_reg_dmn_pairs[] = {
	{NULL1_WORLD, NULL1, WORLD},

	{FCC1_FCCA, FCC1, FCCA},
	{FCC1_WORLD, FCC1, WORLD},
	{FCC2_FCCA, FCC2, FCCA},
	{FCC2_WORLD, FCC2, WORLD},
	{FCC2_ETSIC, FCC2, ETSIC},
	{FCC3_FCCA, FCC3, FCCA},
	{FCC3_WORLD, FCC3, WORLD},
	{FCC3_ETSIC, FCC3, ETSIC},
	{FCC4_FCCA, FCC4, FCCA},
	{FCC5_FCCA, FCC5, FCCA},
	{FCC6_WORLD, FCC6, WORLD},
	{FCC6_FCCA, FCC6, FCCA},
	{FCC8_FCCA, FCC8, FCCA},
	{FCC8_WORLD, FCC8, WORLD},
	{FCC10_FCCA, FCC10, FCCA},
	{FCC11_WORLD, FCC11, WORLD},
	{FCC13_WORLD, FCC13, WORLD},
	{FCC14_FCCB, FCC14, FCCB},
	{FCC14_WORLD, FCC14, WORLD},
	{FCC15_FCCA, FCC15, FCCA},
	{FCC16_FCCA, FCC16, FCCA},
	{FCC17_FCCA, FCC17, FCCA},
	{FCC17_WORLD, FCC17, WORLD},
	{FCC17_ETSIC, FCC17, ETSIC},
	{FCC18_ETSIC, FCC18, ETSIC},
	{FCC20_FCCA, FCC20, FCCA},
	{FCC21_ETSIC, FCC21, ETSIC},

	{ETSI1_WORLD, ETSI1, WORLD},
	{ETSI3_WORLD, ETSI3, WORLD},
	{ETSI4_WORLD, ETSI4, WORLD},
	{ETSI8_WORLD, ETSI8, WORLD},
	{ETSI9_WORLD, ETSI9, WORLD},
	{ETSI10_WORLD, ETSI10, WORLD},
	{ETSI10_FCCA, ETSI10, FCCA},
	{ETSI11_WORLD, ETSI11, WORLD},
	{ETSI12_WORLD, ETSI12, WORLD},
	{ETSI13_WORLD, ETSI13, WORLD},
	{ETSI14_WORLD, ETSI14, WORLD},
	{ETSI15_WORLD, ETSI15, WORLD},
	{ETSI15_ETSIC, ETSI15, ETSIC},
	{ETSI16_WORLD, ETSI16, WORLD},
	{ETSI17_WORLD, ETSI17, WORLD},
	{ETSI18_WORLD, ETSI18, WORLD},
	{ETSI19_WORLD, ETSI19, WORLD},
	{ETSI20_WORLD, ETSI20, WORLD},
	{ETSI21_WORLD, ETSI21, WORLD},

	{APL1_WORLD, APL1, WORLD},
	{APL1_ETSIC, APL1, ETSIC},
	{APL2_WORLD, APL2, WORLD},
	{APL2_ETSIC, APL2, ETSIC},
	{APL2_ETSID, APL2, ETSID},
	{APL4_WORLD, APL4, WORLD},
	{APL6_WORLD, APL6, WORLD},
	{APL8_WORLD, APL8, WORLD},
	{APL9_WORLD, APL9, WORLD},
	{APL9_MKKC, APL9, MKKC},
	{APL9_KRRA, APL9, KRRA},
	{APL10_WORLD, APL10, WORLD},
	{APL11_FCCA, APL11, FCCA},
	{APL12_WORLD, APL12, WORLD},
	{APL13_WORLD, APL13, WORLD},
	{APL14_WORLD, APL14, WORLD},
	{APL14_CHNA, APL14, CHNA},
	{APL15_WORLD, APL15, WORLD},
	{APL16_WORLD, APL16, WORLD},
	{APL16_ETSIC, APL16, ETSIC},
	{APL17_ETSIC, APL17, ETSIC},
	{APL17_ETSID, APL17, ETSID},
	{APL19_ETSIC, APL19, ETSIC},
	{APL20_WORLD, APL20, WORLD},
	{APL23_WORLD, APL23, WORLD},
	{APL24_ETSIC, APL24, ETSIC},
	{APL25_ETSIC, APL25, ETSIC},
	{APL26_ETSIC, APL26, ETSIC},
	{APL27_FCCA, APL27, FCCA},
	{APL28_ETSIC, APL28, ETSIC},
	{APL29_ETSIC, APL29, ETSIC},

	{MKK3_MKKC, MKK3, MKKC},
	{MKK5_MKKA, MKK5, MKKA},
	{MKK5_MKKC, MKK5, MKKC},
	{MKK11_MKKC, MKK11, MKKC},
	{MKK16_MKKC, MKK16, MKKC},
	{MKK17_MKKC, MKK17, MKKC},
	{MKK18_MKKC, MKK18, MKKC},

	{WORLD_60, WORLD_5G_2, WORLD_2G_3},
	{WORLD_61, WORLD_5G_2, WORLD_2G_3},
	{WORLD_62, WORLD_5G_2, WORLD_2G_3},
	{WORLD_63, WORLD_5G_1, WORLD_2G_2},
	{WORLD_65, WORLD_5G_1, WORLD_2G_2},
	{WORLD_64, WORLD_5G_1, WORLD_2G_1},
	{WORLD_66, WORLD_5G_2, WORLD_2G_1},
	{WORLD_69, WORLD_5G_2, WORLD_2G_1},
	{WORLD_67, WORLD_5G_2, WORLD_2G_2},
	{WORLD_68, WORLD_5G_2, WORLD_2G_2},
	{WORLD_6A, WORLD_5G_2, WORLD_2G_2},
	{WORLD_6C, WORLD_5G_2, WORLD_2G_2},
};

enum reg_rules_2g {

	CHAN_1_11_1,
	CHAN_1_11_2,
	CHAN_1_11_3,
	CHAN_1_13_1,
	CHAN_1_13_2,
	CHAN_1_13_3,
	CHAN_1_13_4,
	CHAN_1_13_5,
	CHAN_12_12_1,
	CHAN_12_13_1,
	CHAN_14_1,
	CHAN_14_2,
};

const struct regulatory_rule reg_rules_2g[] = {

	[CHAN_1_11_1] = {2402, 2472, 40, 30, 0},
	[CHAN_1_11_2] = {2402, 2472, 40, 20, 0},
	[CHAN_1_11_3] = {2402, 2472, 40, 36, 0},
	[CHAN_1_13_1] = {2402, 2482, 40, 20, 0},
	[CHAN_1_13_2] = {2402, 2482, 40, 30, 0},
	[CHAN_1_13_3] = {2402, 2482, 40, 36, 0},
	[CHAN_1_13_4] = {2402, 2482, 40, 23, 0},
	[CHAN_1_13_5] = {2402, 2482, 20, 20, 0},
	[CHAN_12_12_1] = {2457, 2477, 20, 20, REGULATORY_CHAN_NO_IR},
	[CHAN_12_13_1] = {2457, 2482, 20, 20, REGULATORY_CHAN_NO_IR},
	[CHAN_14_1] = {2474, 2494, 20, 23, REGULATORY_CHAN_NO_OFDM},
	[CHAN_14_2] = {2474, 2494, 20, 20,
		       REGULATORY_CHAN_NO_OFDM | REGULATORY_CHAN_NO_IR},
};

const struct regdomain regdomains_2g[] = {

	[FCCA] = {CTL_FCC, DFS_UNINIT_REGION, 0, 40, 6, 1, {CHAN_1_11_1} },
	[FCCB] = {CTL_FCC, DFS_UNINIT_REGION, 0, 40, 6, 1, {CHAN_1_11_3} },
	[WORLD] = {CTL_ETSI, DFS_UNINIT_REGION, 0, 40, 0, 1, {CHAN_1_13_1} },
	[MKKA] = {CTL_MKK, DFS_UNINIT_REGION, 0, 40, 0, 2, {CHAN_1_13_4,
							    CHAN_14_1} },
	[MKKC] = {CTL_MKK, DFS_UNINIT_REGION, 0, 40, 0, 1, {CHAN_1_13_4} },
	[ETSIC] = {CTL_ETSI, DFS_UNINIT_REGION, 0, 40, 0, 1, {CHAN_1_13_2} },
	[ETSID] = {CTL_ETSI, DFS_UNINIT_REGION, 0, 20, 0, 1, {CHAN_1_13_5} },
	[KRRA]  = {CTL_KOR, DFS_UNINIT_REGION, 0, 40, 0, 1,  {CHAN_1_13_4} },
	[CHNA]  = {CTL_CHN, DFS_UNINIT_REGION, 0, 40, 0, 1,  {CHAN_1_13_1} },
	[WORLD_2G_1] = {CTL_NONE, DFS_UNINIT_REGION, 0, 40, 0, 1, {CHAN_1_11_2}
								   },
	[WORLD_2G_2] = {CTL_NONE, DFS_UNINIT_REGION, 0, 40, 0, 2, {CHAN_1_11_2,
								   CHAN_12_13_1}
								   },
	[WORLD_2G_3] = {CTL_NONE, DFS_UNINIT_REGION, 0, 40, 0, 2, {CHAN_1_11_2,
								   CHAN_12_12_1}
								   },
};


enum reg_rules_5g {

	CHAN_4910_4990_1,
	CHAN_4940_4990_1,
	CHAN_5030_5090_1,
	CHAN_5170_5250_1,
	CHAN_5170_5250_2,
	CHAN_5170_5250_3,
	CHAN_5170_5250_4,
	CHAN_5170_5250_5,
	CHAN_5170_5250_6,
	CHAN_5170_5250_7,
	CHAN_5170_5250_8,
	CHAN_5170_5250_9,
	CHAN_5170_5250_10,
	CHAN_5170_5250_11,
	CHAN_5170_5250_12,
	CHAN_5170_5250_13,
	CHAN_5170_5250_14,
	CHAN_5170_5330_1,
	CHAN_5170_5330_2,
	CHAN_5170_5330_3,
	CHAN_5250_5330_1,
	CHAN_5250_5330_2,
	CHAN_5250_5330_3,
	CHAN_5250_5330_4,
	CHAN_5250_5330_5,
	CHAN_5250_5330_6,
	CHAN_5250_5330_7,
	CHAN_5250_5330_8,
	CHAN_5250_5330_9,
	CHAN_5250_5330_10,
	CHAN_5250_5330_11,
	CHAN_5250_5330_12,
	CHAN_5250_5330_13,
	CHAN_5250_5330_14,
	CHAN_5250_5330_15,
	CHAN_5250_5330_16,
	CHAN_5250_5330_17,
	CHAN_5250_5330_18,
	CHAN_5490_5730_1,
	CHAN_5490_5730_2,
	CHAN_5490_5730_3,
	CHAN_5490_5730_4,
	CHAN_5490_5730_5,
	CHAN_5490_5730_6,
	CHAN_5490_5730_7,
	CHAN_5490_5730_8,
	CHAN_5490_5730_9,
	CHAN_5490_5710_1,
	CHAN_5490_5710_2,
	CHAN_5490_5710_3,
	CHAN_5490_5710_4,
	CHAN_5490_5710_5,
	CHAN_5490_5710_6,
	CHAN_5490_5710_7,
	CHAN_5490_5710_8,
	CHAN_5490_5710_9,
	CHAN_5490_5590_1,
	CHAN_5490_5590_2,
	CHAN_5490_5590_3,
	CHAN_5490_5590_4,
	CHAN_5490_5650_2,
	CHAN_5490_5670_1,
	CHAN_5490_5670_2,
	CHAN_5490_5630_1,
	CHAN_5650_5730_1,
	CHAN_5650_5730_2,
	CHAN_5650_5730_3,
	CHAN_5650_5730_4,
	CHAN_5735_5835_1,
	CHAN_5735_5835_2,
	CHAN_5735_5835_3,
	CHAN_5735_5835_4,
	CHAN_5735_5835_5,
	CHAN_5735_5835_6,
	CHAN_5735_5835_7,
	CHAN_5735_5835_9,
	CHAN_5735_5835_10,
	CHAN_5735_5835_11,
	CHAN_5735_5875_1,
	CHAN_5735_5875_2,
	CHAN_5735_5875_3,
	CHAN_5735_5875_4,
	CHAN_5735_5875_5,
	CHAN_5735_5875_6,
	CHAN_5735_5875_7,
	CHAN_5735_5875_8,
	CHAN_5735_5815_1,
	CHAN_5735_5815_2,
	CHAN_5735_5815_3,
	CHAN_5735_5815_4,
	CHAN_5735_5775_1,
	CHAN_5735_5895_1,
	CHAN_5735_5895_2,
	CHAN_5835_5855_1,
	CHAN_5835_5855_2,
	CHAN_5835_5895_1,
	CHAN_5850_5925_1,
	CHAN_5850_5925_2,
#if defined(CONFIG_BAND_6GHZ) && defined(COMPILE_REGDB_6G)
	CHAN_5945_7125_1,
	CHAN_5945_7125_2,
	CHAN_5945_7125_3,
#endif
};

const struct regulatory_rule reg_rules_5g[] = {

	[CHAN_4910_4990_1] = {4910, 4990, 20, 20, 0},
	[CHAN_4940_4990_1] = {4940, 4990, 20, 33, 0},
	[CHAN_5030_5090_1] = {5030, 5090, 20, 20, 0},
	[CHAN_5170_5250_1] = {5170, 5250, 80, 17, 0},
	[CHAN_5170_5250_2] = {5170, 5250, 80, 23, 0},
	[CHAN_5170_5250_3] = {5170, 5250, 80, 20, 0},
	[CHAN_5170_5250_4] = {5170, 5250, 80, 30, 0},
	[CHAN_5170_5250_5] = {5170, 5250, 80, 24, 0},
	[CHAN_5170_5250_6] = {5170, 5250, 80, 18, 0},
	[CHAN_5170_5250_7] = {5170, 5250, 80, 20, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5170_5250_8] = {5170, 5250, 80, 23, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5170_5250_9] = {5170, 5250, 40, 30, 0},
	[CHAN_5170_5250_10] = {5170, 5250, 20, 20, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5170_5250_11] = {5170, 5250, 80, 16, 0},
	[CHAN_5170_5250_12] = {5170, 5250, 80, 24, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5170_5250_13] = {5170, 5250, 80, 17, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5170_5250_14] = {5170, 5250, 80, 36, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5170_5330_1] = {5170, 5330, 160, 20, REGULATORY_CHAN_NO_IR},
	[CHAN_5170_5330_2] = {5170, 5330, 160, 24, 0},
	[CHAN_5170_5330_3] = {5170, 5330, 160, 23, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5250_5330_1] = {5250, 5330, 80, 23, REGULATORY_CHAN_RADAR},
	[CHAN_5250_5330_2] = {5250, 5330, 80, 20, REGULATORY_CHAN_RADAR},
	[CHAN_5250_5330_3] = {5250, 5330, 80, 18, REGULATORY_CHAN_RADAR},
	[CHAN_5250_5330_4] = {5250, 5330, 80, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5250_5330_5] = {5250, 5330, 80, 23, 0},
	[CHAN_5250_5330_6] = {5250, 5330, 80, 30, 0},
	[CHAN_5250_5330_7] = {5250, 5330, 80, 24, REGULATORY_CHAN_RADAR},
	[CHAN_5250_5330_8] = {5250, 5330, 80, 36, 0},
	[CHAN_5250_5330_9] = {5250, 5330, 80, 20, 0},
	[CHAN_5250_5330_10] = {5250, 5330, 80, 24, 0},
	[CHAN_5250_5330_11] = {5250, 5330, 80, 20, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5250_5330_12] = {5250, 5330, 80, 23, REGULATORY_CHAN_RADAR |
						   REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5250_5330_13] = {5250, 5330, 40, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5250_5330_14] = {5250, 5330, 80, 20, REGULATORY_CHAN_RADAR |
						   REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5250_5330_15] = {5250, 5330, 20, 20, REGULATORY_CHAN_RADAR |
						   REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5250_5330_16] = {5250, 5330, 80, 23, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5250_5330_17] = {5250, 5330, 80, 30, REGULATORY_CHAN_RADAR |
						   REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5250_5330_18] = {5250, 5330, 80, 16, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5730_1] = {5490, 5730, 160, 24, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5730_2] = {5490, 5730, 160, 20, REGULATORY_CHAN_NO_IR},
	[CHAN_5490_5730_3] = {5490, 5730, 160, 30, 0},
	[CHAN_5490_5730_4] = {5490, 5730, 160, 24, 0},
	[CHAN_5490_5730_5] = {5490, 5730, 160, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5730_6] = {5490, 5730, 160, 23, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5730_7] = {5490, 5730, 160, 20, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5730_8] = {5490, 5730, 160, 30, REGULATORY_CHAN_RADAR |
						   REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5490_5730_9] = {5490, 5730, 80, 30, 0},
	[CHAN_5490_5710_1] = {5490, 5710, 160, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5710_2] = {5490, 5710, 160, 20, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5710_3] = {5490, 5710, 160, 27, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5710_4] = {5490, 5710,  40, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5710_5] = {5490, 5710, 160, 24, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5710_6] = {5490, 5710, 160, 26, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5710_7] = {5490, 5710, 160, 23, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5710_8] = {5490, 5710, 20, 27, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5710_9] = {5490, 5710, 160, 30, REGULATORY_CHAN_RADAR |
						   REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5490_5590_1] = {5490, 5590, 80, 24, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5590_2] = {5490, 5590, 80, 24, 0},
	[CHAN_5490_5590_3] = {5490, 5590, 80, 36, 0},
	[CHAN_5490_5590_4] = {5490, 5590, 80, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5650_2] = {5490, 5650, 160, 24, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5670_1] = {5490, 5670, 160, 20, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5670_2] = {5490, 5670, 160, 23, REGULATORY_CHAN_RADAR},
	[CHAN_5490_5630_1] = {5490, 5630, 80, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5650_5730_1] = {5650, 5730, 80, 24, REGULATORY_CHAN_RADAR},
	[CHAN_5650_5730_2] = {5650, 5730, 80, 24, 0},
	[CHAN_5650_5730_3] = {5650, 5730, 80, 36, 0},
	[CHAN_5650_5730_4] = {5650, 5730, 80, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5735_5835_1] = {5735, 5835, 80, 23, 0},
	[CHAN_5735_5835_2] = {5735, 5835, 80, 30, 0},
	[CHAN_5735_5835_3] = {5735, 5835, 80, 20, 0},
	[CHAN_5735_5835_4] = {5735, 5835, 80, 33, 0},
	[CHAN_5735_5835_5] = {5735, 5835, 80, 20, REGULATORY_CHAN_NO_IR},
	[CHAN_5735_5835_6] = {5735, 5835, 80, 24, 0},
	[CHAN_5735_5835_7] = {5735, 5835, 80, 36, 0},
	[CHAN_5735_5835_9] = {5735, 5835, 80, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5735_5835_10] = {5735, 5835, 80, 14, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5735_5835_11] = {5735, 5835, 80, 27, 0},
	[CHAN_5735_5875_1] = {5735, 5875, 20, 27, REGULATORY_CHAN_RADAR},
	[CHAN_5735_5875_2] = {5735, 5875, 20, 30, 0},
	[CHAN_5735_5875_3] = {5735, 5875, 80, 30, 0},
	[CHAN_5735_5875_4] = {5735, 5875, 80, 14, 0},
	[CHAN_5735_5875_5] = {5735, 5875, 80, 20, REGULATORY_CHAN_RADAR},
	[CHAN_5735_5875_6] = {5735, 5875, 80, 23, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5735_5875_7] = {5735, 5875, 80, 14, REGULATORY_CHAN_RADAR},
	[CHAN_5735_5875_8] = {5735, 5875, 80, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5735_5815_1] = {5735, 5815, 80, 30, 0},
	[CHAN_5735_5815_2] = {5735, 5815, 80, 30, REGULATORY_CHAN_RADAR},
	[CHAN_5735_5815_3] = {5735, 5815, 80, 23, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5735_5815_4] = {5735, 5815, 20, 20, 0},
	[CHAN_5735_5775_1] = {5735, 5775, 40, 23, 0},
	[CHAN_5735_5895_1] = {5735, 5895, 160, 30, 0},
	[CHAN_5735_5895_2] = {5735, 5895, 160, 20, REGULATORY_CHAN_NO_IR},
	[CHAN_5835_5855_1] = {5835, 5855, 20, 30, 0},
	[CHAN_5835_5855_2] = {5835, 5855, 20, 14, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5835_5895_1] = {5835, 5895, 40, 30, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5850_5925_1] = {5850, 5925, 20, 24, 0},
	[CHAN_5850_5925_2] = {5850, 5925, 20, 30, 0},
#if defined(CONFIG_BAND_6GHZ) && defined(COMPILE_REGDB_6G)
	[CHAN_5945_7125_1] = {5945, 7125, 160, 21, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5945_7125_2] = {5945, 7125, 160, 27, REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5945_7125_3] = {5945, 7125, 160, 20, REGULATORY_CHAN_NO_IR},
#endif
};

const struct regdomain regdomains_5g[] = {

	[FCC1] = {CTL_FCC, DFS_FCC_REGION, 2, 160, 6, 3, {CHAN_5170_5250_1,
							  CHAN_5250_5330_1,
							  CHAN_5735_5835_2} },

	[FCC2] = {CTL_FCC, DFS_FCC_REGION, 2, 160, 6, 3, {CHAN_5170_5250_2,
							  CHAN_5250_5330_1,
							  CHAN_5735_5835_2} },

	[FCC3] = {CTL_FCC, DFS_FCC_REGION, 2, 320, 6, 5, {CHAN_5170_5250_5,
							  CHAN_5250_5330_7,
							  CHAN_5490_5730_1,
							  CHAN_5735_5835_2,
							  CHAN_5835_5895_1} },

	[FCC4] = {CTL_FCC, DFS_FCC_REGION, 2, 160, 6, 4, {CHAN_4940_4990_1,
							  CHAN_5170_5250_1,
							  CHAN_5250_5330_7,
							  CHAN_5735_5835_2} },

	[FCC5] = {CTL_FCC, DFS_UNINIT_REGION, 2, 80, 6, 2, {CHAN_5170_5250_4,
							    CHAN_5735_5835_2} },

	[FCC6] = {CTL_FCC, DFS_FCC_REGION, 2, 160, 6, 5, {CHAN_5170_5250_12,
							  CHAN_5250_5330_7,
							  CHAN_5490_5590_1,
							  CHAN_5650_5730_1,
							  CHAN_5735_5835_2} },

	[FCC8] = {CTL_FCC, DFS_FCC_REGION, 2, 320, 6, 5, {CHAN_5170_5250_4,
							  CHAN_5250_5330_7,
							  CHAN_5490_5730_1,
							  CHAN_5735_5835_2,
							  CHAN_5835_5895_1} },

	[FCC10] = {CTL_FCC, DFS_FCC_REGION, 2, 160, 0, 5, {CHAN_5170_5250_4,
							   CHAN_5250_5330_7,
							   CHAN_5490_5730_1,
							   CHAN_5735_5835_2,
							   CHAN_5850_5925_1} },

	[FCC11] = {CTL_FCC, DFS_FCC_REGION, 2, 160, 6, 4, {CHAN_5170_5250_5,
							   CHAN_5250_5330_7,
							   CHAN_5490_5650_2,
							   CHAN_5735_5835_6} },

	[FCC13] = {CTL_FCC, DFS_UNINIT_REGION, 2, 320, 0, 3, {CHAN_5170_5330_2,
							      CHAN_5490_5730_4,
							      CHAN_5735_5835_2}
							      },

	[FCC14] = {CTL_FCC, DFS_UNINIT_REGION, 2, 320, 0, 4, {CHAN_5170_5250_4,
							      CHAN_5250_5330_10,
							      CHAN_5490_5730_4,
							      CHAN_5735_5835_2}
							      },

#if defined(CONFIG_BAND_6GHZ) && defined(COMPILE_REGDB_6G)
	[FCC15] = {CTL_FCC, DFS_FCC_REGION, 2, 160, 0, 6, {CHAN_5170_5250_5,
							   CHAN_5250_5330_7,
							   CHAN_5490_5730_1,
							   CHAN_5735_5835_2,
							   CHAN_5835_5895_1,
							   CHAN_5945_7125_1} },

	[FCC16] = {CTL_FCC, DFS_FCC_REGION, 2, 160, 0, 6, {CHAN_5170_5250_4,
							   CHAN_5250_5330_7,
							   CHAN_5490_5730_1,
							   CHAN_5735_5835_2,
							   CHAN_5835_5895_1,
							   CHAN_5945_7125_2} },
#endif
	[FCC17] = {CTL_FCC, DFS_FCC_REGION, 2, 320, 6, 4, {CHAN_5170_5250_5,
							   CHAN_5250_5330_7,
							   CHAN_5490_5730_1,
							   CHAN_5735_5835_2} },

	[FCC18] = {CTL_FCC, DFS_FCC_REGION, 2, 320, 6, 4, {CHAN_5170_5250_14,
							   CHAN_5250_5330_17,
							   CHAN_5490_5730_1,
							   CHAN_5735_5835_2} },

	[FCC20] = {CTL_FCC, DFS_FCC_REGION, 2, 320, 6, 4, {CHAN_5170_5250_4,
							   CHAN_5250_5330_4,
							   CHAN_5490_5730_5,
							   CHAN_5735_5835_2} },

	[FCC21] = {CTL_FCC, DFS_FCC_REGION, 2, 160, 6, 2, {CHAN_5170_5250_11,
							   CHAN_5250_5330_18} },

	[ETSI1] = {CTL_ETSI, DFS_ETSI_REGION, 2, 160, 0, 3, {CHAN_5170_5250_8,
							     CHAN_5250_5330_12,
							     CHAN_5490_5710_1}
							     },

	[ETSI3] = {CTL_ETSI, DFS_ETSI_REGION, 5, 160, 0, 2, {CHAN_5170_5250_2,
							     CHAN_5250_5330_1}
							     },

	[ETSI4] = {CTL_ETSI, DFS_ETSI_REGION, 2, 160, 0, 2, {CHAN_5170_5250_6,
							     CHAN_5250_5330_3}
							     },

	[ETSI8] = {CTL_ETSI, DFS_UNINIT_REGION, 20, 160, 0, 4,
							{CHAN_5170_5250_2,
							 CHAN_5250_5330_5,
							 CHAN_5490_5730_3,
							 CHAN_5735_5835_2} },

	[ETSI9] = {CTL_ETSI, DFS_ETSI_REGION, 20, 160, 0, 4, {CHAN_5170_5250_2,
							      CHAN_5250_5330_1,
							      CHAN_5490_5710_5,
							      CHAN_5735_5835_6}
							      },

	[ETSI10] = {CTL_ETSI, DFS_ETSI_REGION, 5, 160, 0, 4,
							{CHAN_5170_5250_7,
							 CHAN_5250_5330_14,
							 CHAN_5490_5710_3,
							 CHAN_5850_5925_2} },

	[ETSI11] = {CTL_ETSI, DFS_ETSI_REGION, 10, 20, 0, 4, {CHAN_5170_5250_7,
							      CHAN_5250_5330_14,
							      CHAN_5490_5710_3,
							      CHAN_5735_5875_1}
							      },

	[ETSI12] = {CTL_ETSI, DFS_ETSI_REGION, 2, 320, 0, 4, {CHAN_5170_5250_8,
							      CHAN_5250_5330_12,
							      CHAN_5490_5730_5,
							      CHAN_5735_5835_1}
							      },

	[ETSI13] = {CTL_ETSI, DFS_ETSI_REGION, 2, 160, 0, 4, {CHAN_5170_5250_2,
							      CHAN_5250_5330_12,
							      CHAN_5490_5710_1,
							      CHAN_5735_5875_4}
							      },

	[ETSI14] = {CTL_ETSI, DFS_ETSI_REGION, 2, 320, 0, 4, {CHAN_5170_5250_2,
							      CHAN_5250_5330_1,
							      CHAN_5490_5730_7,
							      CHAN_5735_5875_5}
							      },

	[ETSI15] = {CTL_ETSI, DFS_ETSI_REGION, 2, 320, 0, 4, {CHAN_5170_5250_2,
							      CHAN_5250_5330_1,
							      CHAN_5490_5730_5,
							      CHAN_5735_5815_2}
							      },

	[ETSI16] = {CTL_ETSI, DFS_UNINIT_REGION, 2, 80, 0, 2, {CHAN_5170_5250_11,
							       CHAN_5735_5875_4}
							       },

	[ETSI17] = {CTL_ETSI, DFS_ETSI_REGION, 2, 160, 0, 5, {CHAN_5170_5250_8,
							      CHAN_5250_5330_12,
							      CHAN_5490_5710_9,
							      CHAN_5735_5835_10,
							      CHAN_5835_5855_2}
							      },

	[ETSI18] = {CTL_ETSI, DFS_ETSI_REGION, 2, 320, 0, 4, {CHAN_5170_5250_2,
							      CHAN_5250_5330_12,
							      CHAN_5490_5730_5,
							      CHAN_5735_5875_4}
							      },

	[ETSI19] = {CTL_ETSI, DFS_ETSI_REGION, 2, 160, 0, 5, {CHAN_5170_5250_11,
							      CHAN_5250_5330_12,
							      CHAN_5490_5710_9,
							      CHAN_5735_5835_10,
							      CHAN_5835_5855_2}
							      },

	[ETSI20] = {CTL_ETSI, DFS_ETSI_REGION, 2, 160, 0, 4, {CHAN_5170_5250_8,
							      CHAN_5250_5330_12,
							      CHAN_5490_5710_1,
							      CHAN_5735_5875_4}
							      },

	[ETSI21] = {CTL_ETSI, DFS_ETSI_REGION, 2, 80, 0, 1, {CHAN_5170_5250_11}
							    },

	[APL1] = {CTL_ETSI, DFS_UNINIT_REGION, 2, 80, 0, 1, {CHAN_5735_5835_2}
							     },

	[APL2] = {CTL_ETSI, DFS_UNINIT_REGION, 2, 20, 0, 1, {CHAN_5735_5815_4}
							     },

	[APL4] = {CTL_ETSI, DFS_UNINIT_REGION, 2, 80, 0, 2, {CHAN_5170_5250_2,
							     CHAN_5735_5875_6}
							     },

	[APL6] = {CTL_ETSI, DFS_ETSI_REGION, 2, 160, 0, 3, {CHAN_5170_5250_3,
							    CHAN_5250_5330_2,
							    CHAN_5735_5835_3} },

	[APL8] = {CTL_FCC, DFS_ETSI_REGION, 2, 80, 0, 2, {CHAN_5250_5330_4,
							  CHAN_5735_5835_2} },

	[APL9] = {CTL_KOR, DFS_KR_REGION, 2, 160, 6, 4,   {CHAN_5170_5250_2,
							   CHAN_5250_5330_1,
							   CHAN_5490_5730_6,
							   CHAN_5735_5835_1} },

	[APL10] = {CTL_ETSI, DFS_FCC_REGION, 2, 160, 6, 4, {CHAN_5170_5250_2,
							    CHAN_5250_5330_4,
							    CHAN_5490_5710_1,
							    CHAN_5735_5815_1} },

	[APL11] = { CTL_ETSI, DFS_FCC_REGION, 2, 40, 0, 4, {CHAN_5170_5250_9,
							    CHAN_5250_5330_13,
							    CHAN_5490_5710_4,
							    CHAN_5735_5875_2} },

	[APL12] = {CTL_ETSI, DFS_ETSI_REGION, 2, 320, 0, 4, {CHAN_5170_5250_2,
							    CHAN_5250_5330_16,
							    CHAN_5490_5730_8,
							    CHAN_5735_5875_6} },

	[APL13] = {CTL_ETSI, DFS_ETSI_REGION, 2, 160, 0, 3, {CHAN_5170_5250_2,
							     CHAN_5250_5330_1,
							     CHAN_5490_5670_2}
							     },

	[APL14] = {CTL_CHN, DFS_CN_REGION, 2, 160, 0, 3, {CHAN_5170_5250_8,
							  CHAN_5250_5330_12,
							  CHAN_5735_5835_4} },

	[APL15] = {CTL_FCC, DFS_UNINIT_REGION, 2, 160, 0, 3, {CHAN_5170_5250_2,
							      CHAN_5250_5330_5,
							      CHAN_5735_5835_4}
							      },

	[APL16] = {CTL_FCC, DFS_UNINIT_REGION, 2, 160, 0, 5, {CHAN_5170_5250_1,
							      CHAN_5250_5330_10,
							      CHAN_5490_5590_2,
							      CHAN_5650_5730_2,
							      CHAN_5735_5835_2}
							      },

	[APL17] = {CTL_FCC, DFS_UNINIT_REGION, 2, 160, 0, 5, {CHAN_5170_5250_2,
							      CHAN_5250_5330_8,
							      CHAN_5490_5590_3,
							      CHAN_5650_5730_3,
							      CHAN_5735_5835_7}
							      },

	[APL19] = {CTL_FCC, DFS_FCC_REGION, 2, 320, 0, 4, {CHAN_5170_5250_4,
							   CHAN_5250_5330_7,
							   CHAN_5490_5730_1,
							   CHAN_5735_5875_3} },

	[APL20] = {CTL_ETSI, DFS_ETSI_REGION, 2, 320, 0, 4, {CHAN_5170_5250_8,
							     CHAN_5250_5330_12,
							     CHAN_5490_5730_5,
							     CHAN_5735_5835_4}
							     },

	[APL23] = {CTL_ETSI, DFS_UNINIT_REGION, 2, 160, 0, 3,
							{CHAN_5170_5250_7,
							 CHAN_5250_5330_11,
							 CHAN_5735_5835_3} },

	[APL24] = {CTL_ETSI, DFS_ETSI_REGION, 2, 80, 0, 3, {CHAN_5170_5250_8,
							    CHAN_5250_5330_12,
							    CHAN_5735_5815_3} },

	[APL25] = {CTL_ETSI, DFS_ETSI_REGION, 2, 160, 0, 5, {CHAN_5170_5250_8,
							     CHAN_5250_5330_12,
							     CHAN_5490_5590_4,
							     CHAN_5650_5730_4,
							     CHAN_5735_5835_7}
							    },

	[APL26] = {CTL_ETSI, DFS_ETSI_REGION, 2, 320, 0, 4, {CHAN_5170_5250_8,
							    CHAN_5250_5330_12,
							    CHAN_5490_5730_5,
							    CHAN_5735_5835_7} },

	[APL27] = {CTL_FCC, DFS_ETSI_REGION, 2, 320, 0, 4, {CHAN_5170_5250_13,
							    CHAN_5250_5330_7,
							    CHAN_5490_5730_1,
							    CHAN_5735_5835_2} },

	[APL28] = {CTL_ETSI, DFS_ETSI_REGION, 2, 320, 0, 4, {CHAN_5170_5250_8,
							     CHAN_5250_5330_12,
							     CHAN_5490_5730_6,
							     CHAN_5735_5875_3}
							    },

	[APL29] = {CTL_FCC, DFS_FCC_REGION, 2, 320, 0, 4, {CHAN_5170_5250_2,
							   CHAN_5250_5330_1,
							   CHAN_5490_5730_1,
							   CHAN_5735_5835_11}
							  },

	[MKK3] = {CTL_MKK, DFS_UNINIT_REGION, 2, 80, 0, 1, {CHAN_5170_5250_3} },

	[MKK5] = {CTL_MKK, DFS_MKK_REGION, 2, 160, 0, 3, {CHAN_5170_5250_8,
							  CHAN_5250_5330_12,
							  CHAN_5490_5710_7} },

	[MKK11] = {CTL_MKK, DFS_MKK_REGION, 2, 160, 0, 5, {CHAN_4910_4990_1,
							   CHAN_5170_5250_2,
							   CHAN_5030_5090_1,
							   CHAN_5250_5330_1,
							   CHAN_5490_5710_7} },

	[MKK16] = {CTL_MKK, DFS_MKK_REGION, 2, 160, 0, 1, {CHAN_5490_5710_6} },

	[MKK17] = {CTL_MKK, DFS_MKKN_REGION, 2, 160, 0, 3, {CHAN_5170_5250_8,
							    CHAN_5250_5330_12,
							    CHAN_5490_5730_6} },

	[MKK18] = {CTL_MKK, DFS_MKKN_REGION, 2, 80, 0, 1, {CHAN_5170_5250_11} },

#if defined(CONFIG_BAND_6GHZ) && defined(COMPILE_REGDB_6G)
	[WORLD_5G_1] = {CTL_NONE, DFS_UNINIT_REGION, 2, 160, 0, 3, {CHAN_5170_5330_1,
								    CHAN_5735_5835_5,
								    CHAN_5945_7125_3}
								    },

	[WORLD_5G_2] = {CTL_NONE, DFS_UNINIT_REGION, 2, 160, 0, 4,
							{CHAN_5170_5330_1,
							 CHAN_5490_5730_2,
							 CHAN_5735_5895_2,
							 CHAN_5945_7125_3}
							},
#else
	[WORLD_5G_1] = {CTL_NONE, DFS_UNINIT_REGION, 2, 160, 0, 2, {CHAN_5170_5330_1,
								    CHAN_5735_5835_5}
								    },

	[WORLD_5G_2] = {CTL_NONE, DFS_UNINIT_REGION, 2, 160, 0, 3,
							{CHAN_5170_5330_1,
							 CHAN_5490_5730_2,
							 CHAN_5735_5895_2} },
#endif

};

#if defined(CONFIG_BAND_6GHZ) && defined(COMPILE_REGDB_6G)
/*
 * List of 6G Sub Domains.
 *
 * Note - If not specified in the naming, then above 6G subdomains are for
 * both AP and STA (eg ETSI1_VLP_6G).
 */
enum reg_subdomains_6g {
	FCC1_CLI_LPI_DEFAULT_6G = 0x01,
	FCC1_CLI_SP_6G = 0x02,
	FCC1_AP_LPI_6G = 0x03,
	FCC1_CLI_LPI_SUBORDINATE = FCC1_AP_LPI_6G,
	FCC1_AP_SP_6G = 0x04,
	ETSI1_AP_LPI_6G = 0x10,
	ETSI1_CLI_LPI_6G = ETSI1_AP_LPI_6G,
	FCC2_AP_SP_6G = 0x05,
	FCC2_CLI_SP_6G = 0x07,
	FCC3_AP_VLP_6G = 0x06,
	FCC3_CLI_VLP_6G =  FCC3_AP_VLP_6G,
	ETSI1_AP_VLP_6G = 0x11,
	ETSI1_CLI_VLP_6G = ETSI1_AP_VLP_6G,
	ETSI2_AP_LPI_6G = 0x12,
	ETSI2_CLI_LPI_6G = ETSI2_AP_LPI_6G,
	ETSI2_AP_VLP_6G = 0x13,
	ETSI2_CLI_VLP_6G = ETSI2_AP_VLP_6G,
	APL1_AP_LPI_6G = 0x20,
	APL1_CLI_LPI_6G = APL1_AP_LPI_6G,
	APL1_AP_VLP_6G = 0x21,
	APL1_CLI_VLP_6G = APL1_AP_VLP_6G,
	APL2_AP_LPI_6G = 0x22,
	APL2_CLI_LPI_6G = APL2_AP_LPI_6G,
	APL2_AP_VLP_6G = 0x23,
	APL2_CLI_VLP_6G = APL2_AP_VLP_6G,
	APL3_AP_VLP_6G = 0x24,
	APL3_CLI_VLP_6G = APL3_AP_VLP_6G,
	APL3_AP_LPI_6G = 0x25,
	APL3_CLI_LPI_6G = APL3_AP_LPI_6G,
	APL4_AP_LPI_6G = 0x26,
	APL4_CLI_LPI_6G = APL4_AP_LPI_6G,
	APL4_AP_VLP_6G = 0x27,
	APL4_CLI_VLP_6G = APL4_AP_VLP_6G,
	APL5_AP_LPI_6G = 0x32,
	APL5_AP_VLP_6G = 0x28,
	APL5_CLI_VLP_6G = APL5_AP_VLP_6G,
	APL5_CLI_LPI_DEFAULT_6G = 0x31,
	APL6_CLI_LPI_6G = 0x29,
	APL6_AP_LPI_6G = 0x30,
	MKK1_AP_LPI_6G = 0x60,
	MKK1_CLI_LPI_6G = MKK1_AP_LPI_6G,
	MKK1_AP_VLP_6G = 0x61,
	MKK1_CLI_VLP_6G = MKK1_AP_VLP_6G,
};

/*
 * Table of 6G super domain to sub domain.
 */
static const struct sixghz_super_to_subdomains g_6g_reg_dmn_9_tuples[] = {
	{FCC1_6G_01, FCC1_AP_LPI_6G, 0, 0,
	 {FCC1_CLI_LPI_DEFAULT_6G, FCC1_CLI_LPI_SUBORDINATE},
	 {FCC1_CLI_SP_6G, FCC1_CLI_SP_6G},
	 {0, 0} },
	{ETSI1_6G_02, ETSI1_AP_LPI_6G, 0, ETSI1_AP_VLP_6G,
	 {ETSI1_CLI_LPI_6G, ETSI1_CLI_LPI_6G},
	 {0, 0},
	 {ETSI1_CLI_VLP_6G, ETSI1_CLI_VLP_6G} },
	{ETSI2_6G_03, ETSI2_AP_LPI_6G, 0, ETSI2_AP_VLP_6G,
	 {ETSI2_CLI_LPI_6G, ETSI2_CLI_LPI_6G},
	 {0, 0},
	 {ETSI2_CLI_VLP_6G, ETSI2_CLI_VLP_6G} },
	{APL1_6G_04, APL1_AP_LPI_6G, 0, APL1_AP_VLP_6G,
	 {APL1_CLI_LPI_6G, APL1_CLI_LPI_6G},
	 {0, 0},
	 {APL1_CLI_VLP_6G, APL1_CLI_VLP_6G} },
	{FCC1_6G_05, FCC1_AP_LPI_6G, 0, 0,
	 {FCC1_CLI_LPI_DEFAULT_6G, FCC1_CLI_LPI_SUBORDINATE},
	 {0, 0},
	 {0, 0} },
	{APL2_6G_06, APL2_AP_LPI_6G, 0, APL2_AP_VLP_6G,
	 {APL2_CLI_LPI_6G, APL2_CLI_LPI_6G},
	 {0, 0},
	 {APL2_CLI_VLP_6G, APL2_CLI_VLP_6G} },
	{FCC1_6G_07, FCC1_AP_LPI_6G, 0, APL3_AP_VLP_6G,
	 {FCC1_CLI_LPI_DEFAULT_6G, 0},
	 {0, 0},
	 {APL3_CLI_VLP_6G, APL3_CLI_VLP_6G} },
	{APL3_6G_08, APL3_AP_LPI_6G, 0, 0,
	 {APL3_CLI_LPI_6G, APL3_CLI_LPI_6G},
	 {0, 0},
	 {0, 0} },
	{FCC1_6G_09, FCC1_AP_LPI_6G, FCC1_AP_SP_6G, 0,
	 {FCC1_CLI_LPI_DEFAULT_6G, 0},
	 {FCC1_CLI_SP_6G, FCC1_CLI_SP_6G},
	 {0, 0} },
	{APL6_6G_0A, APL6_AP_LPI_6G, 0, 0,
	 {APL6_CLI_LPI_6G, APL6_CLI_LPI_6G},
	 {0, 0},
	 {0, 0} },
	{MKK1_6G_0B, MKK1_AP_LPI_6G, 0, MKK1_AP_VLP_6G,
	 {MKK1_CLI_LPI_6G, MKK1_CLI_LPI_6G},
	 {0, 0},
	 {MKK1_CLI_VLP_6G, MKK1_CLI_VLP_6G} },
	{ETSI2_6G_0C, ETSI2_AP_LPI_6G, 0, ETSI1_AP_VLP_6G,
	 {ETSI2_CLI_LPI_6G, ETSI2_CLI_LPI_6G},
	 {0, 0},
	 {ETSI1_CLI_VLP_6G, ETSI1_CLI_VLP_6G} },
	{ETSI1_6G_0D, ETSI1_AP_LPI_6G, 0, 0,
	 {ETSI1_CLI_LPI_6G, ETSI1_CLI_LPI_6G},
	 {0, 0},
	 {0, 0} },
	{ETSI2_6G_0E, ETSI2_AP_LPI_6G, 0, ETSI1_AP_VLP_6G,
	 {ETSI2_CLI_LPI_6G, ETSI2_CLI_LPI_6G},
	 {0, 0},
	 {ETSI1_CLI_VLP_6G, ETSI1_CLI_VLP_6G} },
	{FCC2_6G_10, FCC1_AP_LPI_6G, FCC2_AP_SP_6G, 0,
	 {FCC1_CLI_LPI_DEFAULT_6G, 0},
	 {FCC2_CLI_SP_6G, FCC2_CLI_SP_6G},
	 {0, 0} },
	{APL4_6G_11, APL4_AP_LPI_6G, 0, APL4_AP_VLP_6G,
	 {APL4_CLI_LPI_6G, APL4_CLI_LPI_6G},
	 {0, 0},
	 {APL4_CLI_VLP_6G, APL4_CLI_VLP_6G} },
	{APL5_6G_12, FCC1_AP_LPI_6G, 0, APL5_AP_VLP_6G,
	 {FCC1_CLI_LPI_DEFAULT_6G, 0},
	 {0, 0},
	 {APL5_CLI_VLP_6G, APL5_CLI_VLP_6G} },
	{APL7_6G_13, APL5_AP_LPI_6G, 0, APL5_AP_VLP_6G,
	 {APL5_CLI_LPI_DEFAULT_6G, 0},
	 {0, 0},
	 {APL5_CLI_VLP_6G, APL5_CLI_VLP_6G} },
	{FCC1_6G_14, FCC1_AP_LPI_6G, 0, FCC3_AP_VLP_6G,
	 {FCC1_CLI_LPI_DEFAULT_6G, FCC1_CLI_LPI_SUBORDINATE},
	 {FCC1_CLI_SP_6G, FCC1_CLI_SP_6G},
	 {FCC3_CLI_VLP_6G, FCC3_CLI_VLP_6G} },
};

/*
 * List of 6G reg rules.
 */
enum reg_rules_6g {
	CHAN_5925_6425_1,
	CHAN_5925_6425_2,
	CHAN_5925_6425_3,
	CHAN_5925_6425_4,
	CHAN_5925_6425_5,
	CHAN_5925_6425_6,
	CHAN_5925_6425_7,
	CHAN_5925_6425_8,
	CHAN_5925_6425_9,
	CHAN_5925_6425_10,
	CHAN_5925_6425_11,
	CHAN_5925_6425_12,
	CHAN_5925_6425_13,
	CHAN_5925_6425_14,
	CHAN_5925_6425_15,
	CHAN_5945_6425_1,
	CHAN_5945_6425_2,
	CHAN_5945_6425_3,
	CHAN_5945_6425_4,
	CHAN_5945_6425_5,
	CHAN_5945_6425_6,
	CHAN_5945_6425_7,
	CHAN_5945_6425_8,
	CHAN_5945_6425_9,
	CHAN_5945_6425_10,
	CHAN_5945_6425_11,
	CHAN_6425_6525_1,
	CHAN_6425_6525_2,
	CHAN_6425_6525_3,
	CHAN_6425_6525_4,
	CHAN_6425_6525_5,
	CHAN_6425_6525_6,
	CHAN_6425_6525_7,
	CHAN_6425_6525_8,
	CHAN_6425_6525_9,
	CHAN_6425_6525_10,
	CHAN_6425_6525_11,
	CHAN_6425_6525_12,
	CHAN_6425_6525_13,
	CHAN_6525_6875_1,
	CHAN_6525_6875_2,
	CHAN_6525_6875_3,
	CHAN_6525_6875_4,
	CHAN_6525_6875_5,
	CHAN_6525_6875_6,
	CHAN_6525_6875_7,
	CHAN_6525_6875_8,
	CHAN_6525_6875_9,
	CHAN_6525_6875_10,
	CHAN_6525_6875_11,
	CHAN_6525_6875_12,
	CHAN_6525_6875_13,
	CHAN_6525_6875_14,
	CHAN_6525_6875_15,
	CHAN_6875_7125_1,
	CHAN_6875_7125_2,
	CHAN_6875_7125_3,
	CHAN_6875_7125_4,
	CHAN_6875_7125_5,
	CHAN_6875_7125_6,
	CHAN_6875_7125_7,
	CHAN_6875_7125_8,
	CHAN_6875_7125_9,
	CHAN_6875_7125_10,
	CHAN_6875_7125_11,
};

/*
 * Mapping of 6G reg rules present in the 6G sub domain.
 */
static const struct regulatory_rule_ext reg_rules_6g[] = {
	[CHAN_5925_6425_1] = {5925, 6425, 320, 23, REG_MAX_PSD,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5925_6425_2] = {5925, 6425, 320, 17, -5, 0},
	[CHAN_5925_6425_3] = {5925, 6425, 320, 24, REG_MAX_PSD,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5925_6425_4] = {5925, 6425, 320, 30, 17, 0},
	[CHAN_5925_6425_5] = {5925, 6425, 320, 30, 5,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5925_6425_6] = {5925, 6425, 320, 24, -1,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5925_6425_7] = {5925, 6425, 320, 36, 23, 0},
	[CHAN_5925_6425_8] = {5925, 6425, 320, 30, REG_MAX_PSD,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5925_6425_9] = {5925, 6425, 320, 14, REG_MAX_PSD, 0},
	[CHAN_5925_6425_10] = {5925, 6425, 320, 17, -1, 0},
	[CHAN_5925_6425_11] = {5925, 6425, 320, 24, 10,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5925_6425_12] = {5925, 6425, 320, 30, 10,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5925_6425_13] = {5925, 6425, 160, 23, 10,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5925_6425_14] = {5925, 6425, 160, 23, 10, 0},
	[CHAN_5925_6425_15] = {5925, 6425, 320, 14, -8, 0},
	[CHAN_5945_6425_1] = {5945, 6425, 320, 30, 5,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5945_6425_2] = {5945, 6425, 320, 36, 23, 0},
	[CHAN_5945_6425_3] = {5945, 6425, 320, 30, 17, 0},
	[CHAN_5945_6425_4] = {5945, 6425, 320, 23, 10,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5945_6425_5] = {5945, 6425, 320, 24, -1,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5945_6425_6] = {5945, 6425, 320, 14, 1, 0},
	[CHAN_5945_6425_7] = {5945, 6425, 320, 24, 11,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5945_6425_8] = {5945, 6425, 320, 14, 11, 0},
	[CHAN_5945_6425_9] = {5945, 6425, 160, 63, 2,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_5945_6425_10] = {5945, 6425, 160, 14, 1, 0},
	[CHAN_6425_6525_1] = {6425, 6525, 320, 30, 5,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6425_6525_2] = {6425, 6525, 320, 24, -1,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6425_6525_3] = {6425, 6525, 100, 63, 2,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6425_6525_4] = {6425, 6525, 320, 23, REG_MAX_PSD,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6425_6525_5] = {6425, 6525, 320, 17, -5, 0},
	[CHAN_6425_6525_6] = {6425, 6525, 320, 36, 23, 0},
	[CHAN_6425_6525_7] = {6425, 6525, 320, 30, REG_MAX_PSD,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6425_6525_8] = {6425, 6525, 320, 14, REG_MAX_PSD, 0},
	[CHAN_6425_6525_9] = {6425, 6525, 320, 17, -1, 0},
	[CHAN_6425_6525_10] = {6425, 6525, 320, 24, 10,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6425_6525_11] = {6425, 6525, 320, 30, 10,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6425_6525_12] = {6425, 6525, 320, 14, -8, 0},
	[CHAN_6425_6525_13] = {6425, 6525, 320, 30, 17, 0},
	[CHAN_6525_6875_1] = {6525, 6875, 320, 30, 5,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6525_6875_2] = {6525, 6865, 160, 36, 23, 0},
	[CHAN_6525_6875_3] = {6525, 6865, 160, 30, 17, 0},
	[CHAN_6525_6875_4] = {6525, 6875, 320, 24, -1,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6525_6875_5] = {6525, 6875, 160, 63, 2,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6525_6875_6] = {6525, 6875, 320, 22, REG_MAX_PSD,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6525_6875_7] = {6525, 6875, 160, 17, -5, 0},
	[CHAN_6525_6875_8] = {6525, 6875, 320, 36, 23, 0},
	[CHAN_6525_6875_9] = {6525, 6875, 320, 30, REG_MAX_PSD,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6525_6875_10] = {6525, 6875, 320, 14, REG_MAX_PSD, 0},
	[CHAN_6525_6875_11] = {6525, 6875, 320, 17, -1, 0},
	[CHAN_6525_6875_12] = {6525, 6875, 320, 24, 10,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6525_6875_13] = {6525, 6875, 320, 30, 10,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6525_6875_14] = {6525, 6875, 320, 14, -8, 0},
	[CHAN_6525_6875_15] = {6525, 6765, 320, 30, 17, 0},
	[CHAN_6875_7125_1] = {6875, 7125, 160, 30, 5,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6875_7125_2] = {6875, 7125, 160, 24, -1,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6875_7125_3] = {6875, 7125, 160, 63, 2,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6875_7125_4] = {6875, 7125, 160, 22, REG_MAX_PSD,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6875_7125_5] = {6875, 7125, 320, 17, -5, 0},
	[CHAN_6875_7125_6] = {6875, 7125, 320, 30, REG_MAX_PSD,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6875_7125_7] = {6875, 7125, 320, 14, REG_MAX_PSD, 0},
	[CHAN_6875_7125_8] = {6875, 7125, 320, 17, -1, 0},
	[CHAN_6875_7125_9] = {6875, 7125, 160, 24, 10,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6875_7125_10] = {6875, 7125, 160, 30, 10,
						REGULATORY_CHAN_INDOOR_ONLY},
	[CHAN_6875_7125_11] = {6875, 7125, 320, 14, -8, 0},
};

/*
 * Mapping of 6G sub domain and 6G reg rules.
 */
static const struct sub_6g_regdomain sub_regdomains_6g[] = {
	[FCC1_AP_LPI_6G] = {2, 320, 4, {CHAN_5925_6425_5,
					CHAN_6425_6525_1,
					CHAN_6525_6875_1,
					CHAN_6875_7125_1} },
	[FCC1_AP_SP_6G] = {2, 320, 2, {CHAN_5925_6425_7,
				       CHAN_6525_6875_2} },
	[FCC1_CLI_SP_6G] = {2, 320, 2, {CHAN_5925_6425_4,
					CHAN_6525_6875_3} },
	[FCC1_CLI_LPI_DEFAULT_6G] = {2, 320,  4, {CHAN_5925_6425_6,
						  CHAN_6425_6525_2,
						  CHAN_6525_6875_4,
						  CHAN_6875_7125_2} },
	[FCC2_AP_SP_6G] = {2, 320, 3, {CHAN_5925_6425_7,
				       CHAN_6425_6525_6,
				       CHAN_6525_6875_8} },
	[FCC2_CLI_SP_6G] = {2, 320, 3, {CHAN_5925_6425_4,
				       CHAN_6425_6525_13,
				       CHAN_6525_6875_15} },
	[FCC3_AP_VLP_6G] = {2, 320, 4, {CHAN_5925_6425_15,
				       CHAN_6425_6525_12,
				       CHAN_6525_6875_14,
				       CHAN_6875_7125_11} },

	[ETSI1_AP_LPI_6G] = {2, 320, 1, {CHAN_5945_6425_5} },
	[ETSI1_AP_VLP_6G] = {2, 320, 1, {CHAN_5945_6425_6} },
	[ETSI2_AP_LPI_6G] = {2, 320, 1, {CHAN_5945_6425_7} },
	[ETSI2_AP_VLP_6G] = {2, 320, 1, {CHAN_5945_6425_8} },
	[APL1_AP_LPI_6G]  = {2, 160, 4, {CHAN_5945_6425_9,
					 CHAN_6425_6525_3,
					 CHAN_6525_6875_5,
					 CHAN_6875_7125_3} },
	[APL1_AP_VLP_6G]  = {2, 160, 1, {CHAN_5945_6425_10} },
	[APL2_AP_LPI_6G]  = {2, 320, 4, {CHAN_5925_6425_1,
					 CHAN_6425_6525_4,
					 CHAN_6525_6875_6,
					 CHAN_6875_7125_4} },
	[APL2_AP_VLP_6G]  = {2, 320, 4, {CHAN_5925_6425_1,
					 CHAN_6425_6525_4,
					 CHAN_6525_6875_6,
					 CHAN_6875_7125_4} },
	[APL3_AP_VLP_6G]  = {2, 320, 4, {CHAN_5925_6425_2,
					 CHAN_6425_6525_5,
					 CHAN_6525_6875_7,
					 CHAN_6875_7125_5} },
	[APL3_AP_LPI_6G]  = {2, 320, 1, {CHAN_5925_6425_3} },
	[APL4_AP_LPI_6G]  = {2, 320, 4, {CHAN_5925_6425_8,
					 CHAN_6425_6525_7,
					 CHAN_6525_6875_9,
					 CHAN_6875_7125_6} },
	[APL4_AP_VLP_6G]  = {2, 320, 4, {CHAN_5925_6425_9,
					 CHAN_6425_6525_8,
					 CHAN_6525_6875_10,
					 CHAN_6875_7125_7} },
	[APL5_AP_LPI_6G]  = {2, 320, 1, {CHAN_5925_6425_5} },
	[APL5_AP_VLP_6G]  = {2, 320, 1, {CHAN_5925_6425_10} },
	[APL5_CLI_LPI_DEFAULT_6G] = {2, 320,  1, {CHAN_5925_6425_6} },
	[APL6_CLI_LPI_6G]  = {2, 320, 4, {CHAN_5925_6425_11,
					 CHAN_6425_6525_10,
					 CHAN_6525_6875_12,
					 CHAN_6875_7125_9} },
	[APL6_AP_LPI_6G]  = {2, 320, 4, {CHAN_5925_6425_12,
					 CHAN_6425_6525_11,
					 CHAN_6525_6875_13,
					 CHAN_6875_7125_10} },
	[MKK1_AP_LPI_6G]  = {2, 160, 1, {CHAN_5925_6425_13} },
	[MKK1_AP_VLP_6G]  = {2, 160, 1, {CHAN_5945_6425_10} },
};
#endif
#ifdef CONFIG_REG_CLIENT
const uint32_t reg_2g_sub_dmn_code[REG_DOMAINS_2G_MAX] = {
	[FCCA] = 0x0A10,
	[FCCB] = 0x0B90,
	[WORLD] = 0x0199,
	[MKKA] = 0x0A40,
	[MKKC] = 0x0A50,
	[ETSIC] = 0x0C30,
	[ETSID] = 0x0F30,
	[KRRA] = 0x0A60,
	[CHNA] = 0x0A80,
};

const uint32_t reg_5g_sub_dmn_code[REG_DOMAINS_5G_MAX] = {
	[NULL1] = 0x0198,
	[FCC1] = 0x0110,
	[FCC2] = 0x0120,
	[FCC3] = 0x0160,
	[FCC4] = 0x0165,
	[FCC5] = 0x0510,
	[FCC6] = 0x0610,
	[FCC8] = 0x0810,
	[FCC10] = 0x0B10,
	[FCC11] = 0x0B20,
	[FCC13] = 0x0B60,
	[FCC14] = 0x0B70,
	[FCC17] = 0x0B73,
	[FCC18] = 0x0B74,
	[FCC20] = 0x0B75,
	[FCC21] = 0x0B76,
	[ETSI1] = 0x0130,
	[ETSI3] = 0x0330,
	[ETSI4] = 0x0430,
	[ETSI8] = 0x0830,
	[ETSI9] = 0x0930,
	[ETSI10] = 0x0D30,
	[ETSI11] = 0x0E30,
	[ETSI12] = 0x0E38,
	[ETSI13] = 0x0E39,
	[ETSI14] = 0x0E40,
	[ETSI15] = 0x0E41,
	[ETSI18] = 0x0E44,
	[ETSI19] = 0x0E45,
	[ETSI20] = 0x0E46,
	[ETSI21] = 0x0E47,
	[APL1] = 0x0150,
	[APL2] = 0x0250,
	[APL4] = 0x0450,
	[APL6] = 0x0650,
	[APL8] = 0x0850,
	[APL9] = 0x0950,
	[APL10] = 0x1050,
	[APL11] = 0x1150,
	[APL12] = 0x1160,
	[APL13] = 0x1170,
	[APL14] = 0x1180,
	[APL15] = 0x1190,
	[APL16] = 0x1200,
	[APL17] = 0x1210,
	[APL19] = 0x1240,
	[APL20] = 0x1250,
	[APL23] = 0x1280,
	[APL28] = 0x1233,
	[APL29] = 0x1234,
	[MKK3] = 0x0340,
	[MKK5] = 0x0540,
	[MKK11] = 0x1140,
	[MKK16] =  0x1640,
	[MKK17] =  0x1650,
	[MKK18] =  0x1660,
};
#endif

QDF_STATUS reg_get_num_countries(int *num_countries)
{
	*num_countries = QDF_ARRAY_SIZE(g_all_countries);

	return QDF_STATUS_SUCCESS;
}

QDF_STATUS reg_get_num_reg_dmn_pairs(int *num_reg_dmn)
{
	*num_reg_dmn = QDF_ARRAY_SIZE(g_reg_dmn_pairs);

	return QDF_STATUS_SUCCESS;
}

bool reg_etsi_regdmn(uint8_t reg_dmn)
{
	return ((reg_dmn == ETSI13) ||
		(reg_dmn == ETSI18) ||
		(reg_dmn == ETSI20));
}

bool reg_fcc_regdmn(uint8_t reg_dmn)
{
	return ((reg_dmn == FCC3) ||
		(reg_dmn == FCC8) ||
		(reg_dmn == FCC15) ||
		(reg_dmn == FCC16));
}

#ifdef WLAN_REG_PARTIAL_OFFLOAD
QDF_STATUS reg_get_default_country(uint16_t *default_country)
{
	*default_country = CTRY_UNITED_STATES;

	return QDF_STATUS_SUCCESS;
}

bool reg_en302_502_regdmn(uint16_t regdmn)
{
	return ((regdmn == ETSI11_WORLD) ||
		(regdmn == ETSI12_WORLD) ||
		(regdmn == ETSI14_WORLD) ||
		(regdmn == ETSI15_WORLD));
}
#endif
